package javamath.hasse.latdraw;

/* Diagram.java  96/30/6


*/


import java.util.*;
import java.io.*;
import javamath.hasse.lattice.*;
import javamath.hasse.util.*;


public class Diagram 
{

  OrderedSet poset;
  Point3D[] vertices;
  int size;
  // I didn't have 3 and 5 before. 
  static int[] primes = {3, 5, 7, 11, 13, 17, 19, 23, 29, 31};
  int primePointer;
  static float attractionFactorConstant = (float)0.1;
  static float repulsionFactorConstant = (float)1.0;
  //static float repulsionFactorConstant = (float)0.8;
  float attractionFactor;
  float repulsionFactor;
  //static int numberOfIterations = 40;
  static int numberOfIterations = 30;

  // AS: adding <name>
  private String name;

  public String getName()
  {
    return name;
  }

  // Constructors (and methods to help)

  // AS: added <name> 
  public Diagram(String n, Vector labels, Vector ucs) 
    throws NonOrderedSetException {
    //System.out.println("AS: Diagram: name = "+n+"\n");
    poset = new OrderedSet(n, labels, ucs);
    setupDiagram();
  }

  // AS: added <name>
  public Diagram(InputLattice in) throws NonOrderedSetException {
    this(in.name, in.labels, in.upperCoversVector);
  }

  public Diagram(OrderedSet set) throws NonOrderedSetException {
    poset = set;
    setupDiagram();
  }

  void setupDiagram () {
    float PI = (float)Math.PI;
    // AS: adding <name>
    name = poset.getName();

    primePointer = -1;
    size = poset.card();
    attractionFactor = attractionFactorConstant / (float)Math.sqrt(size);
    repulsionFactor = repulsionFactorConstant / (float)Math.sqrt(size);
    vertices = new Point3D[size];
    Vector elems = poset.univ();
    for(int i=0; i < size; i++) {
      vertices[i] = new Point3D(((POElem)elems.elementAt(i)).label());
    }
    Point3D pt;
    int rank;
    for(int i=0; i < size; ) {
      
      pt = vertices[i];
      rank = ((POElem)elems.elementAt(i)).rank();
      // j will be the nunber of vertices with the same rank as the ith one.
      int j;
      for(j=1; j < size - i; j++) {
        if (rank != ((POElem)elems.elementAt(i+j)).rank()) break;
      }
      float angle = 2*PI/j;
      for (int k=0; k < j; k++) {
        //int prime = nextPrime();
        pt = vertices[i+k];
        pt.z = rank;
        //pt.x = j*(float)Math.cos(k*angle + PI/prime);
        //pt.y = j*(float)Math.sin(k*angle + PI/prime);
        pt.x = j*(float)Math.cos(k*angle + PI/nextPrime());
        pt.y = j*(float)Math.sin(k*angle + PI/nextPrime());
      }
      i += j;
    }
  }
      
  // public methods

  public OrderedSet getOrderedSet () {
    return poset;
  }

  public int size () {
    return size;
  }

  public Point3D[] getVertices() {
    return vertices;
  }

  // methods

  int nextPrime() {
    primePointer++;
    if (primePointer == primes.length) primePointer = 0;
    return primes[primePointer];
  }

  // This finds the attraction between two points and updates their currentForce.
  void attraction (Point3D pt1, Point3D pt2, float att_fac) {
    float dx = att_fac * (pt2.x - pt1.x); 
    float dy = att_fac * (pt2.y - pt1.y); 
    pt1.adjustForce(dx, dy);
    pt2.adjustForce(-dx, -dy);
  }

  // This finds the repulsion between two points and updates their currentForce.
  void repulsion (Point3D pt1,  Point3D pt2, float repulsion_fac) {
    float dx = pt1.x - pt2.x;
    float dy = pt1.y - pt2.y;
    int   dz = pt1.z - pt2.z;
    float inv_d_cubed;
    if (dz==0 && -0.2<dx && dx<0.2 && -0.2<dy && dy<0.2) {
      inv_d_cubed = (float)37.0;
    } else {
      inv_d_cubed = (float)1.0 / ((float)Math.pow(Math.abs(dx),3) + 
          (float)Math.pow(Math.abs(dy),3) + 
          (float)Math.pow(Math.abs(dz),3));
    }
    dx *= inv_d_cubed * repulsion_fac;
    dy *= inv_d_cubed * repulsion_fac;
    pt1.adjustForce(dx, dy);
    pt2.adjustForce(-dx, -dy);
  }

  // public methods

  public void multipleUpdates (int k, float att, float repulsion) {
    for(int i=0;i<k;i++) update(att, repulsion);
  }

  public synchronized void update (float att, float repulsion) {
    Enumeration list;
    for (int i = 0; i < size - 1; i++) {
      POElem x = (POElem)poset.univ().elementAt(i);
      list = x.filter().elements();
      list.nextElement();    // Skip the first element which is x.
      while (list.hasMoreElements()) {
        attraction(vertices[i], 
    vertices[poset.elemOrder((POElem)list.nextElement())], att);
      }
      list = x.highIncomparables().elements();
      while (list.hasMoreElements()) {
        repulsion(vertices[i], 
    vertices[poset.elemOrder((POElem)list.nextElement())], repulsion);
      }
    }
    for (int i = 0; i < size; i++) vertices[i].update();
  }

  // This does a translation in the x-y plane so that 0 is at the origin.
  // It returns a scale factor which can be used to get the coords in [0,1].

  protected synchronized float center () {
    Point3D p = vertices[0];
    float x0 = p.x;
    float y0 = p.y;
    float distSq;
    float maxSq = (float)0.0;
    for (int i=0; i < size; i++) {
      p = vertices[i];
      p.x -= x0;
      p.y -= y0;
      distSq = p.x * p.x + p.y * p.y;
      if (distSq > maxSq) maxSq = distSq;
    }
    float maxZ = (float)vertices[size-1].z;
    return (maxZ * maxZ > 4*maxSq ? maxZ : 2*(float)Math.sqrt(maxSq) );
  }


  protected synchronized void normalizeCoords (float scaleFactor) {
    if (scaleFactor != 0) {
      for (int i=0; i < size; i++) vertices[i].setNormalizedCoords(scaleFactor);
    }
  }

  protected synchronized void project2d (float angle) {
    float half = (float)0.5;
    Point3D p;
    for (int i = 0; i < size; i++) {
      p = vertices[i];
      p.project2d(angle);
      p.getProjection().x += half;
    }
  }


  public static void main(String[] args) 
          throws FileNotFoundException, IOException,NonOrderedSetException 
  {
    OrderedSet test;
    if (args.length == 0) 
    {
      //System.out.println("I need the name of a lattice file");
      //System.exit(1);
      // String str = "( (0 (b c)) (a (1)) (b (1)) (c (a)) (1 ()))";
      // test = new OrderedSet(new InputLattice(str,false));
      // a test lattice
      String n = "Andrew";
      Vector l = new Vector();
      l.addElement("0"); 
      l.addElement("a");
      l.addElement("b");
      l.addElement("c");
      l.addElement("topelt bigname");

      Vector c = new Vector();
      Vector tmp;

      // covers of 0
      tmp = new Vector();
      tmp.addElement("b");
      tmp.addElement("c");
      c.addElement(tmp);

      // covers of a
      tmp = new Vector();
      tmp.addElement("topelt bigname");
      c.addElement(tmp);

      // covers of b
      tmp = new Vector();
      tmp.addElement("topelt bigname");
      c.addElement(tmp);


      // covers of c
      tmp = new Vector();
      tmp.addElement("a");
      c.addElement(tmp);

      // covers of 1
      tmp = new Vector();
      c.addElement(tmp);

      InputLattice i = new InputLattice(n, l, c);
      test = new OrderedSet(i);

    } 
    else 
    {
      test = new OrderedSet(new InputLattice(args[0]));
    }
    ChainDecomposition chainDec = new ChainDecomposition(test);
    Enumeration e = test.univ().elements();
    while (e.hasMoreElements()) 
    {
      System.out.print(" " + ((POElem)e.nextElement()).label());
    }
    Diagram testDiagram = new Diagram(test);
    System.out.println("AS:Test diagram's name:"+testDiagram.getName()+"\n");
    System.out.println("\nveritces: ");
    int size = test.card();
    for(int i = 0; i < size; i++) 
    {
      System.out.println("  " + (Point3D)testDiagram.vertices[i]);
    }
    System.out.println("att and rep fac are " + testDiagram.attractionFactor + 
      " " +  testDiagram.repulsionFactor);

    for (int i = 0; i < testDiagram.size + numberOfIterations; i++) 
    {
      testDiagram.update((float)0.5 * testDiagram.attractionFactor, 
      (float)3.0 * testDiagram.repulsionFactor);
      //System.out.println("\nveritces after " + i + ": ");
      //for (int j = 0; j < size; j++) {
      //System.out.println("  " + (Point3D)testDiagram.vertices[j]);
      //}
    }

    for (int i = 0; i < testDiagram.size + numberOfIterations; i++) 
    {
      testDiagram.update((float)3.0 * testDiagram.attractionFactor, 
      (float)0.5 * testDiagram.repulsionFactor);
      //System.out.println("\nveritces after " + i + ": ");
      //System.out.println("  " + (Point3D)testDiagram.vertices[0]);
    }

    for (int i = 0; i < testDiagram.size + 4*numberOfIterations; i++) 
    {
      testDiagram.update(testDiagram.attractionFactor, 
      testDiagram.repulsionFactor);
      //System.out.println("\nveritces after " + i + ": ");
      //System.out.println("  " + (Point3D)testDiagram.vertices[0]);
      //for(int k = 0; k < size; k++) {
      //System.out.println("  " + (Point3D)testDiagram.vertices[k]);
      //}
    }

    float scale = testDiagram.center();
    testDiagram.normalizeCoords(scale);
    testDiagram.project2d(0);

    for(int k = 0; k < size; k++) 
    {
      System.out.println("  " + (Point3D)testDiagram.vertices[k]);
    }  

  }
}

