package javamath.hasse.latdraw;
/* LatDrawWin.java (c) Ralph Freese 96/08/10

The bottons are controlled from their panel but for the improveButton,
labelled "Next", it would be better if I extended the Button class
to ImproveButton and controlled the action form there since I could
then do a disable(). Under the present scheme I need to keep a boolean,
improving, to tell if the button should be active. (I still call
enable() and disable() since they make it grep.

I experimented with popup message (Dialog) windows but first you
need to find the parent frame which is a little tricky for netscape.
The class ralph.util.FindParentFrame does this but netscape makes 
such a big stink about the security that it is annoying so I think
I will use my message area instead.

Window layout. the window hierarchy is a little complicated so it is
nice to have a diagram:

                       LatFrame extends Frame
                        /         \
                  MenuBar      LatDrawWin exents Applet
                              /     |        \
                   LatDrawPanel  Bottom   LatChoose   all extend Panel
                     Center       South     East
                                 /   \
                      ForceControls  ControlsAndMessage extends   Panel
                         North               South
                                         /      |     \
                                   Controls  CountDown  MessageArea
                                    Panel      Canvas     Canvas
                                    North      Center     South
                                      |
                                  several buttons

The MessageArea is changed into an URLPanel (and back) by manipulating
the ControlsAndMessage panel.

Note it would be a little simpler to have all of the stuff below
ControlsAndMessage directly below it using a FlowLayout(5,1).

Most of the important variables are held in LatDrawPanel.


*/

//import javamath.hasse.config.*;
import javamath.hasse.lattice.*;
import javamath.hasse.util.*;
//import javamath.server.proxystore.*;
import java.awt.*;
import java.applet.*;
import java.io.*;
import java.util.*;
import java.net.URL;
import java.awt.event.*;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;




public class LatDrawWin extends Applet {


  /** 
   * Configuration: PROXY_STORE is the url alias of the 
   * ProxyStore on this server.
   */

  boolean isStandalone = false;
  // rsf new
  boolean includeExampleLattices = true;
  boolean showSteps = true;
  //int stepSize = 10;  // only relevant when showSteps is true.
  int stepSize = 2;  // only relevant when showSteps is true.

  // when this is false there will be no Input button
  boolean includeInput = false;

  String file;
  LatDrawPanel dp;
  MessageArea messageArea;
  Controls controls;
  String[][] messages = {
    {"Initial Diagram",
    //"This is the initial positioning of the points in three space.",
    //             "Press Next when you are ready.",
    "Press Improve to impove the diagram.",
    "You can also rotate it at any time.",
		"Mouse over vertex for label"},
	
    {"Phase I: Strong Repulsive Forces",
    "This is the phase where the replusive force is strong.",
    "Press Improve to get the phase where the attractive force is strong.",
    "(The attraction and replusion sliders will only work after Phase III.)",
		"Mouse over vertex for label"},

    {"Phase II: Strong Attractive Forces",
    "This is the phase where the attrative force is strong.",
    "Press Improve to get the phase where the forces are more balanced.",
    "(The attraction and replusion sliders will only work after Phase III.)",
		"Mouse over vertex for label"},

    {"Phase III: Balanced Forces",
    "In this final phase the forces are balanced.",
    "Press one of the rotate buttons to get a better picture.",
    "The Improve button will now apply the forces determined by the sliders.",
		"Mouse over vertex for label"},

    {"Choose your own Forces",
    "The Improve button will now apply the forces determined by the sliders.",
    "Press one of the rotate buttons to get a better picture.",
		"Mouse over vertex for label"} };

  public String[][] getParameterInfo()
  {
    String[][] info = {
      // Parameter Name     Kind of Value   Description
      {"inputType",     "String",     "can be url or cookie"},
      {"inputKey",     "String",     "the url or cookie"}
    };
    return info;
  }

  public void setStandalone(boolean v) {
    isStandalone = v;
  }

  // rsf new
  /**
   * This decides whether to show the examples. The default is to not
   * show them if a lattice was input (in one of our several ways)
   * and to show them otherwise. This makes it possible to override
   * this behaviour.
   */
  public void setIncludeExampleLattices(boolean v) {
    includeExampleLattices = v;
  }

  public void setFile(String v) {
    file = v;
  }

  public String getFile() {
    return file;
  }

  public String getElementAt(int x, int y) {
    Point3D p = dp.getVertexAt(x, y);
    if (p == null) return null;
    return p.getLabel();
  }


  public void start() {
    setLayout(new BorderLayout());
    //boolean getFile = false;
    boolean badOrderedSet = false;
    int reps = Diagram.numberOfIterations;
    // I am setting this to null since otherwise the compiler thinks it may
    // not be initialized.
    Diagram diagram = null;
    String str;
    InputLattice in = null;

    /* When we're coming in through main */
    if (isStandalone && (file != null)) // in this case we look for a file
    {
      try 
      {
        in = new InputLattice(file, InputLattice.FILE);
      } 
      catch (FileNotFoundException e) 
      {
        System.out.println(file + ": file not found.");
        badOrderedSet = true;
      } 
      catch (IOException e) 
      {
        throw new InternalError(e.toString());
      }
    }
    // in this case we look for a url or cookie - came as parameter to applet

    String inputType = null;
    String inputKey = null;
    if (!isStandalone) {
      inputType = getParameter("inputType");
      inputKey = getParameter("inputKey");
    }
    if (!isStandalone && (inputType!= null) && (inputKey!=null))
    {

      if (new String(inputType).equals(new String("url")))
      {

        try 
        {
          in = new InputLattice(inputKey, InputLattice.URL);
        } 
        catch (IOException e) 
        {
          throw new InternalError(e.toString());
        }
      } 
/*
      else {  // inputType == cookie
        URL documentBase = this.getDocumentBase();

        try {
          URL proxyStoreURL = new URL("http", documentBase.getHost(),
          documentBase.getPort(),"/"+ Config.PROXY_STORE);
          ProxyStore proxyStore = new ProxyStore(proxyStoreURL.toString());

          in = (InputLattice) proxyStore.get(inputKey);
        }
        catch (Exception e) {
           System.err.println("Exception: " + e.getMessage() + "\n");
           e.printStackTrace();
        }
      }
*/
    }

    if (in != null) 
    {

      if (! badOrderedSet) 
      {
        try 
        {
          dp = new LatDrawPanel(new Diagram(in), includeExampleLattices);
        } 
        catch (NonOrderedSetException e) 
        {
          System.out.println(e.getMessage());
          /* Couldn't make sense of the input, just proceed as though it 
          ** weren't there.
          */
          dp = new LatDrawPanel(includeExampleLattices);
        }
      }
    }
    else
    {
      dp = new LatDrawPanel(includeExampleLattices);
    }
    
    dp.latDrawWin = this;


    //textarea = new TextArea(messages[0],5,60);
    messageArea = new MessageArea(messages);
    messageArea.resize(dp.bounds().width,120);
    controls = new Controls(dp, messageArea);
    dp.controls = controls;
    add("Center", dp);
    CountDown cd = new CountDown(showSteps);
    dp.countDown = cd;
    cd.resize(dp.bounds().width,4);
    //add("South", new ControlsAndMessage(controls, messageArea, cd));
    add("South", new Bottom(dp, controls, messageArea,
                             cd, new GetURLPanel(dp), new IntervalPanel(dp)));
    LatChooser latChooser = new LatChooser(dp,controls,messageArea);
    //latChooser.resize(220, dp.bounds().height);
    dp.latChooser = latChooser;
    add("East", latChooser);
    this.repaint();

    // rsf new. I decided to make the user click the Improve button once
    //          so that he would have the option of not doing it for very
    //          large lattices.
/*
    messageArea.nextMessage();
    messageArea.repaint();
    dp.forceControls.setForces(dp.att_I,dp.rep_I);
    dp.countDown.resetParameters(dp.phase1);
    Thread fixIt = new ImproveThread(dp);
    dp.improveThread = fixIt;
    fixIt.start();
*/

//Frame fr = FindParentFrame.getFrame(this);
//InfoDialog info = new InfoDialog(fr, "Testing", "This is a test.");
//info.show();

  }

  // This is to stop the running thread and free the resources.
  // Since the improveThread only runs a few seconds, don't worry about it. 
  // Since run isn't implemented, these don't work (?)

  public void stop() {
    dp.rotateStop();    
    controls.setButtonLabels();
  }
 
  /**
   * Properties that fire events.
   */
  public static final String EDGE_CLICKED = "EdgeClicked";
  public static final String EDGE_RIGHT_CLICKED = "EdgeRightClicked";
  public static final String VERTEX_CLICKED = "VertexClicked";
  public static final String VERTEX_RIGHT_CLICKED = "VertexRightClicked";
  public static final String DIAGRAM_CHANGED = "DiagramChanged";


  /**
   * Add property listeners.
   */
  public void addPropertyChangeListener(PropertyChangeListener l) {
    dp.listeners.addPropertyChangeListener(l);
  }

  /**
   * Remove property listeners.
   */
  public void removePropertyChangeListener(PropertyChangeListener l) {
    dp.listeners.removePropertyChangeListener(l);
  }

  // rsf new 

  /**
   * This allows outsiders to set the color map.
   */
  public void setEdgeColorMap(Hashtable ht) {
    dp.edgeColorMap = ht;
  }

  /**
   * This allows outsiders to set the background color.
   */
  public void setBackgroundColor(Color c) {
    dp.backgroundColor = c;
  }

  /**
   * Allow outside parties to see if the current diagram is a subinterval
   * of another diagram.
   */
  public boolean isInterval() {
    return dp.diagram.isInterval();
  }

  private boolean disallowOutsideAction = false;

  public void setDisallowOutsideAction(boolean v) { 
    disallowOutsideAction = v; 
  }

  /**
   * Mark the vertex with <code>label</code> to be colored black. 
   * @returns the previous value or null.
   */
  public boolean addFilledVertex(String label) {
    return addFilledVertex(label, Color.black);
  }

  /**
   * Mark the vertex with <code>label</code> to be colored with 
   * <code>color</code>.
   * @returns true if successful.
   */
  public boolean addFilledVertex(String label, Color color) {
    if (disallowOutsideAction) return false;
    dp.filledVertices.put(label, color);
    return true;
  }
  
  /**
   * Unmakark the vertex with <code>label</code> to be colored. 
   * @returns the value this key had or null.
   */
  public boolean removeFilledVertex(String label) {
    if (disallowOutsideAction) return false;
    dp.filledVertices.remove(label);
    return true;
  }
  
  /**
   * Unmakark all vertices to be colored. 
   */
  public boolean clearFilledVertices() {
    if (disallowOutsideAction) return false;
    dp.filledVertices.clear();
    return true;
  }
  
  /**
   * Mark the vertex with <code>label</code> to be drawn black. 
   * @returns the previous value or null.
   */
  public boolean addDrawnVertex(String label, boolean thick) {
    return addDrawnVertex(label, Color.black, thick);
  }

  /**
   * Mark the vertex with <code>label</code> to be drawn with 
   * <code>color</code>.
   * @returns the previous value or null.
   */
  public boolean addDrawnVertex(String label, Color color, boolean thick) {
    if (disallowOutsideAction) return false;
    Vector v = new Vector();
    v.addElement(color);
    v.addElement( thick ? Boolean.TRUE : Boolean.FALSE );
    dp.drawnVertices.put(label, v);
    return true;
  }
  
  /**
   * Unmakark the vertex with <code>label</code> to be drawn. 
   * @returns the value this key had or null.
   */
  public boolean removeDrawnVertex(String label) {
    if (disallowOutsideAction) return false;
    dp.drawnVertices.remove(label);
    return true;
  }
  
  /**
   * Unmakark all vertices to be drawn. 
   */
  public boolean clearDrawnVertices() {
    if (disallowOutsideAction) return false;
    dp.drawnVertices.clear();
    return true;
  }
  

  /**
   * Add <code>labelsVector</code> (which is a 
   * Vector of two Strings) to the highlightedEdges.
   */
  public boolean addHighlightedEdge(Vector labelsVector) {
    if (disallowOutsideAction) return false;
    dp.highlightedEdges.put(labelsVector, Boolean.TRUE);
    dp.repaint();
    return true;
  }

  /**
   * Remove <code>labelsVector</code> (which is a 
   * Vector of two Strings) to the highlightedEdges.
   */
  public boolean removeHighlightedEdge(Vector labelsVector) {
    if (disallowOutsideAction) return false;
    dp.highlightedEdges.remove(labelsVector);
    dp.repaint();
    return true;
  }

  /**
   * Clear the highlighted edge.
   */
  public boolean clearHighlightedEdges() {
    if (disallowOutsideAction) return false;
    dp.highlightedEdges.clear();
    dp.repaint();
    return true;
  }

  public static void main(String args[]) {
    Frame f = new LatFrame("LatDraw");
    LatDrawWin latdrawwin = new LatDrawWin();
    latdrawwin.isStandalone = true;
    if (args.length != 0) latdrawwin.file = args[0];
    latdrawwin.init();
    latdrawwin.start();
    f.add("Center", latdrawwin);


//Frame fr = FindParentFrame.getFrame(latdrawwin);
//InfoDialog info = new InfoDialog(fr, "Testing", "This is a test.");
//info.show();
    
    MenuBar menubar = new MenuBar();
    f.setMenuBar(menubar);
    Menu file = new Menu("File");
    file.add(new MenuItem("Open"));
    file.add(new MenuItem("Quit"));
    menubar.add(file);
    Menu help = new Menu("Help");
    help.add(new MenuItem("About"));
    menubar.add(help);
    menubar.setHelpMenu(help);


    f.resize(500, 650);
    f.show();
  }

}

class LatDrawPanel extends Panel {


  /**
   * We use this to let outside listeners know about certain events to
   * avoid some proplems with the old stye java events used here.
   */
  PropertyChangeSupport listeners = new PropertyChangeSupport(this);
  
  Vector allLats; //AS: remove artificial distinction between user and builtin

  // rsf new

  Hashtable highlightedEdges = new Hashtable();

  // This is a map from the vertex labels to Colors (if defined at all)
  Hashtable filledVertices = new Hashtable();

  // This is a map from the vertex labels to a Vector whose
  // first component is a Colors and second is a Boolean indicate if
  // the stoke should be thick. If a vertex is not on this table it
  // is just drawn in the usual black.
  Hashtable drawnVertices = new Hashtable();

  boolean includeExampleLattices = true;

  // rsf new
  // static String[] latstr = new String[0];

  static String[] latstr = {
       // N_5
       "(N5 (0 (b c)) (a (1)) (b (1)) (c (a)) (1 ()))",
 
       // D_2
       "(D2 (0 (a b c)) (a (d)) (b (d e)) (c (e)) (d (1)) (e (1)) (1 ()))",
 
       // FD(3)
       "(FD3 (0 (a b c)) (a (d e)) (c (d f)) (b (e f)) (d (x g)) (e (y g)) " +
       "(f (z g)) (g (h i j)) (h (k l)) (i (k m)) (j (l m)) (k (1)) " +
       "(l (1)) (m (1)) (x (h)) (y (i)) (z (j)) (1 ()))",
 
       // FD(3)
/*
       "(FD3 (0 (\"x meet y\" \"x meet z\" \"y meet z\")) " +
            "(\"x meet y\" (\"x meet (x join z)\" \"y meet (x join z)\")) " +
            "(\"x meet z\" (\"x meet (y join z)\" \"z meet (x join y)\")) " +
            "(\"y meet z\" (\"y meet (x join z)\" \"z meet (x join y)\")) " +
            "(\"x meet (y join z)\" " +
                "(\"x\" \"(x meet y) join (x meet y) join (y meet z)\" )) " +
            "(\"y meet (x join z)\" " +
                "(\"y\" \"(x meet y) join (x meet y) join (y meet z)\" )) " +
            "(\"z meet (x join z)\" " +
                "(\"z\" \"(x meet y) join (x meet y) join (y meet z)\" )) " +
            "(\"(x meet y) join (x meet y) join (y meet z)\" " +
                "( i j)) " +
            "(\"x join (y meet z)\" (\"x join y\" \"x join z\")) " +
            "(\"y join (x meet z)\" (\"x join y\" \"y join z\")) " +
            "(\"z join (x meet y)\" (\"x join z\" \"y join z\")) " +
            "(\"x join y\" (1)) " +
            "(\"x join y\" (1)) " +
            "(\"x join y\" (1)) " +
            "(x (\"x join (y meet z)\")) " +
            "(y (\"y join (x meet z)\")) " +
            "(z (\"z join (x meet y)\")) " +
            "(1 ()))",
*/
 
       // FL(1+3)
       "(FL1+3 (0 (2 1 )) (1 (4 )) (2 (4 3 )) (3 (8 6 )) (4 (5 )) (5 (8 7 )) " +
       "(6 (15 )) (7 (10 9 )) (8 (9 )) (9 (11 )) (10 (13 )) " +
       "(11 (13 12 )) (12 (15 14 )) (13 (14 )) (14 (16 )) " +
       "(15 (18 )) (16 (18 17 )) (17 (19 )) (18 (19 )) (19 ()))",
 
       // FM(3)
       "(FM3 (a ( b c ) ) (b ( d ) ) (d ( e ) ) (f ( w j ) ) (u ( l ) ) " +
       "(w ( u ) ) (v ( i ) ) (y ( x n ) ) (x ( v ) ) (1 ( ) ) (h ( 1 ) ) " +
       "(k ( 1 ) ) (g ( 1 ) ) (i ( g k ) ) (l ( g h ) ) (e ( h k ) ) " +
       "(c ( m d ) ) (j ( m u ) ) (n ( m v ) ) (m ( e i l ) ) " +
       "(o ( c n j ) ) (p ( o a ) ) (q ( o f ) ) (s ( o y ) ) (t ( p s ) ) " +
       "(z (s q ) ) (r ( p q ) ) (0 ( r z t ) ))",
 
 
       // BA(3)
       "(BA3 (0 (a b c)) (a (ab ac)) (b (ab bc)) (c (ac bc)) (ab (1)) " +
       "(ac (1)) (bc (1)) (1 ()))",
 
       // BA(4)
       "(BA4 (0 (1 2 4 8)) (1 (3 5 9)) (2 (3 6 10)) (3 (7 11)) (4 (5 6 12)) " + 
       "(5 (7 13)) (6 (7 14)) (7 (15)) (8 (9 10 12)) (9 (11 13)) " + 
       "(10 (11 14)) (11 (15)) (12 (13 14)) (13 (15)) (14 (15)) (15 ()))"};

  static float rotAngleDelta = (float)Math.PI/30;
  static int reps = Diagram.numberOfIterations;

  LatDrawWin latDrawWin;
  Diagram diagram; 
  OrderedSet poset;
  int size;
  int pointRadius = 5;
  int pointDiameter = 2 * pointRadius;
  int improvementCount;
  int phase1, phase2, phase3;
  float rotationAngle;
  Controls controls;
  CountDown countDown;
  ForceControls forceControls;     // set when Bottom is initiated.
  ControlsAndMessage controlsAndMessage;
  LatChooser latChooser;
  Thread rotateThread;
  Thread improveThread;
  boolean rotating;
  boolean improving;
  //boolean erase = false;    // used to unpaint
  float attFactor;
  float repFactor;
  float att_I;        // attraction in Phase I, etc.
  float att_II;
  float att_III;
  float att_user;      // user set attraction.
  float rep_I;
  float rep_II;
  float rep_III;
  float rep_user;      // user set repulsion.

  /* Values for converting mouseEvents into vertex coordinates */
  int wd, ht; //store the nomimal dimensions of the panel
  int xorig, yorig; 

  Color backgroundColor = new Color(160, 160, 160);

  // point labels
  String pntLabel = null;
  Graphics tg = null;
   Point3D currentVertex = null;
  boolean justChanged = false; // from in to out or out to in
  int currentMouseX = 0;
  int currentMouseY = 0;

  /**
   * Let the "listeners" know something was clicked.
   */
  public boolean mouseDown(Event e, int x, int y) {
    String edgePropName = latDrawWin.EDGE_CLICKED;
    String vertexPropName = latDrawWin.VERTEX_CLICKED;
    if ((e.modifiers & Event.META_MASK) != 0) {
      edgePropName = latDrawWin.EDGE_RIGHT_CLICKED;
      vertexPropName = latDrawWin.VERTEX_RIGHT_CLICKED;
    }
    Point3D p = getVertexAt(x, y);
    Point3D[] edge = getEdgeAt(x,y);
    if (edge != null) {
      listeners.firePropertyChange(edgePropName, null, 
                     new String[] {edge[0].getLabel(), edge[1].getLabel()});
      //System.out.println("edge " + edge[0].getLabel() 
                                 //+ ", " + edge[1].getLabel());
    }
    if (p != null) {
      listeners.firePropertyChange(vertexPropName, null, p.getLabel());
    }
    return true;
  }


  public boolean mouseMove(Event e, int x, int y)
  {
    Point3D tmpv = vertexAt(x - xorig, y - yorig);
    // if we were on no vertex or are changing vertex...
    if (tmpv != null && ((currentVertex == null) || 
          !(new String(currentVertex.getLabel()).equals(tmpv.getLabel()) )))
    {
      currentVertex = tmpv;
      currentMouseX = x;
      currentMouseY = y;
      justChanged = true;
      repaint();
    } 
    // if we go from some vertex to no vertex ...
    else if ((tmpv == null) && (currentVertex != null))
    {
      currentVertex = null;
      justChanged = true;
      repaint();
    }
    else
    {
      justChanged = false;
    }
    return true;
  }

  public LatDrawPanel() 
  {
    this(true);
  }

  public LatDrawPanel(boolean includeExamples) 
  {
    // rsf new
    includeExampleLattices = includeExamples;

    //setBackground(Color.white);
    // rsf new
    setBackground(backgroundColor);

    allLats = new Vector();
    if (includeExampleLattices)
    {
      for (int i=0; i<latstr.length;i++)
      {
        try
        {
          allLats.addElement(new Diagram(
            new InputLattice(latstr[i],InputLattice.STRING)));
        }
        catch(java.io.FileNotFoundException e)
        {
          throw new InternalError(e.toString());
        }
        catch(javamath.hasse.lattice.NonOrderedSetException e)
        {
          throw new InternalError(e.toString());
        }
        catch(java.io.IOException e)
        {
          throw new InternalError(e.toString());
        }
      }
      setup((Diagram) allLats.elementAt(2));
    }
  }

  public LatDrawPanel(Diagram d) {
    this(d, false);
  }

  public LatDrawPanel(Diagram d, boolean includeExamples) {
    this(includeExamples);
    allLats.addElement(d);
    setup(d);
  }

  public Diagram stringToDiagram(String s) 
  {
    InputLattice in;
    Diagram d;

    /* AS: Assign to "in" the InputLattice resulting from the string "s".
    ** "s" could be a filename, url or lattice encoded in LatDraw format.
    */
    try 
    {
      in = new InputLattice(s, false);
    } 
    catch (FileNotFoundException e) 
    {
      throw new InternalError(e.toString());
    } 
    catch (IOException e) 
    {
      throw new InternalError(e.toString());
    }


    // AS: Convert the InputLattice to a Diagram
    try 
    {
      d = new Diagram(in);
    } 
    catch (NonOrderedSetException e) 
    {    
      // can't occur now; change later
      throw new InternalError(e.toString());
    }
    return d;
  }

  public void setup (String s) {
    setup(stringToDiagram(s));
  }

  public void setup (Diagram d) {
    setup(d, false);
  }

  /**
   * Setup the diagram in the drawing panel. 
   * 
   * @param isNew indicates the user just did something to generate this.
   */
  public void setup (Diagram d, boolean isNew) {
    rotating = false;
    improving = false;
    if (rotateThread != null) {
      rotateThread.stop();
      rotateThread = null;
    }
    if (improveThread != null) {
      improveThread.stop();
      improveThread = null;
    }
    diagram = d;
    float scale = diagram.center();
    diagram.normalizeCoords(scale);
    diagram.project2d(diagram.getRotationAngle());
    poset = d.getOrderedSet();
    // fire a PropertyChange event
    Vector v = new Vector();
    v.addElement(d.isInterval() ? Boolean.TRUE : Boolean.FALSE);
    v.addElement(poset.zero().label());
    v.addElement(poset.one().label());
    v.addElement(isNew ? Boolean.TRUE : Boolean.FALSE);
    listeners.firePropertyChange(latDrawWin.DIAGRAM_CHANGED, null, v);
    size = d.size();
    rotationAngle = diagram.getRotationAngle();
    improvementCount = diagram.getImprovementCount();
    phase1 = reps + size + 30;
    phase2 = phase1 + reps + size;
    phase3 = phase2 + reps + size + 20;
    attFactor = d.attractionFactor;
    repFactor = d.repulsionFactor;
    att_user = attFactor;
    rep_user = repFactor;
    att_I = (float)0.5 * attFactor;
    att_II = (float)3.0 * attFactor;  // was 3.0
    att_III = (float)0.75 * attFactor;  // was 1.0
    rep_I = (float)3.0 * repFactor;
    rep_II = (float)0.5 * repFactor;  // was 0.5
    rep_III = (float)1.5 * repFactor;  // was 1.0

    this.repaint();
  }

  public synchronized void paint(Graphics g) {
    Rectangle r = this.bounds();


    int ht = (int)Math.round(0.95 * r.height);
    int wd = (int)Math.round(0.95 * r.width);


    this.wd=wd;
    this.ht=ht;
    this.xorig = (int)Math.round(0.025 * r.width);
    this.yorig = (int)Math.round(0.025 * r.height);

    g.translate(xorig,yorig);

    g.setColor(getForeground());
    paintCovers(g, wd, ht);
    paintElems(g, wd, ht, false);
    if (justChanged)
    {
      paintLabel(g, wd, ht);
    }

  }


  public void paintLabel(Graphics g, int wd, int ht) 
  {
    Rectangle r = this.bounds();
    if (currentVertex != null)
    {
      int textwd = 
        g.getFontMetrics().getMaxAdvance()*currentVertex.getLabel().length();
      int textht = getFont().getSize();

      g.setColor(getForeground());
      if (tg != null)
        tg.dispose();
      tg = g.create(Math.min(currentMouseX,r.width-textwd),Math.max(currentMouseY - 20,0), 
        (int) Math.round(textwd*1.2)+5, (int) Math.round(textht*1.2));
      tg.setColor(Color.yellow);
      tg.fillRect(0,0, textwd+5, (int) Math.round(textht*1.1));
      tg.setColor(getForeground());
      tg.drawRect(0,0, textwd+5, (int) Math.round(textht*1.1));
      tg.setPaintMode();
      tg.drawString(currentVertex.getLabel(), 5,textht);
    }
    else
    {
      g.setColor(getBackground());
      if (tg != null)
        tg.dispose();
      g.setColor(getForeground());
    }
  }

  public void paintCovers(Graphics g, int wd, int ht) {
    POElem elem;
    Point3D pt, pt2;
    Point3D[] vertices = diagram.getVertices();
    Vector elems = poset.univ();
    Hashtable edgeColors = diagram.getEdgeColors();
    int x,y,x2,y2;
    for (int i = 0; i < size; i++) {
      elem = (POElem)elems.elementAt(i);
      Enumeration covs = ((Vector)elem.upperCovers()).elements();
      pt = vertices[i];
      x = convertx(pt.getX(), wd);
      y = converty(pt.getY(), ht);
      while (covs.hasMoreElements()) {
        pt2 = vertices[poset.elemOrder((POElem)covs.nextElement())];
        x2 = convertx(pt2.getX(), wd);
        y2 = converty(pt2.getY(), ht);
        Vector v = new Vector(2);
        v.addElement(pt.getLabel());
        v.addElement(pt2.getLabel());
        if (edgeColors != null) {
          String colorStr = (String)edgeColors.get(v);
          if (colorStr != null) g.setColor(getEdgeColor(colorStr));
        }
        g.drawLine(x,y,x2,y2);
        if (highlightedEdges.containsKey(v)) {
          g.drawLine(x - 1, y, x2 - 1, y2);
          g.drawLine(x + 1, y, x2 + 1, y2);
        }
        g.setColor(getForeground());
      }
    }
  }

  Hashtable edgeColorMap;

  // also add a way to import one
  public void setupEdgeColorMap() {
    edgeColorMap = new Hashtable();
    edgeColorMap.put("1", new Color(255,0,0));
    edgeColorMap.put("2", new Color(255,255,0));
    edgeColorMap.put("3", new Color(255,255,255));
    edgeColorMap.put("4", new Color(0,0,255));
    edgeColorMap.put("5", new Color(0,0,0));
  }

  public Color getEdgeColor(String str) {
    if (edgeColorMap == null) setupEdgeColorMap();
    Color c = (Color)edgeColorMap.get(str);
    if (c != null) return c;
    return getForeground();
  }

  public void paintElems(Graphics g,int wd, int ht, boolean erase) {
    int n = size;
    Point3D[] vertices = diagram.getVertices();
    for (int i = 0; i < n; i++) {
      if (erase) eraseElem(g, vertices[i], wd, ht);
      else paintElem(g, vertices[i], wd, ht);
    }
  }

  /**
   * Return the edge at <i>x</i>, <i>y</i> for the raw mouse 
   * <i>x</i> and <i>y</i>.
   */
  public Point3D[] getEdgeAt(int xc, int yc) {
    long x = (long)(xc - xorig);
    long y = (long)(yc - yorig);
    final long epsilon = 7;  // 7 pixels; maybe pointRadius*2 or something
    POElem elem;
    Point3D pt, pt2;
    Point3D[] vertices = diagram.getVertices();
    Vector elems = poset.univ();
    long x1,y1,x2,y2;
    for (int i = 0; i < size; i++) {
      elem = (POElem)elems.elementAt(i);
      pt = vertices[i];
      y1 = (long)converty(pt.getY(), ht);
      if (y >  y1 - epsilon) continue;  // these y's are down based
      x1 = (long)convertx(pt.getX(), wd);
      Enumeration covs = ((Vector)elem.upperCovers()).elements();
      while (covs.hasMoreElements()) {
        pt2 = vertices[poset.elemOrder((POElem)covs.nextElement())];
        y2 = (long)converty(pt2.getY(), ht);
        if (y <  y2 + epsilon) continue;  // these y's are down based
        x2 = (long)convertx(pt2.getX(), wd);
        long dx = x2 - x1;
        long dy = y2 - y1;
        long u = dx * (y - y1) - dy * (x - x1);
        if (u * u < 25 * (dx * dx + dy * dy)) {
          return new Point3D[] {pt, pt2};
        }
      }
    }
    return null;
  }
  
  /**
   * Return the vertex at <i>x</i>, <i>y</i> for the raw mouse 
   * <i>x</i> and <i>y</i>.
   */
  public Point3D getVertexAt(int x, int y) {
    return vertexAt(x - xorig, y - yorig);
  }
  

  /** 
   * Take the rectangle with upper left at (xmin,ymin) and
   * and return the first point centre is in that rectangle
   * under the current projection.
   */
  public Point3D vertexAt(int xmid, int ymid)
  {
    int xmin = xmid-pointRadius;
    int ymin = ymid-pointRadius;
    int xmax = xmid + pointDiameter;
    int ymax = ymid + pointDiameter;
    Point3D[] vertices = diagram.getVertices();

    // Need the dimensions of this panel   ---rsf why?
    // Rectangle r = this.bounds();

    for (int i = 0; i < vertices.length; i++) 
    { 
      Point3D p = vertices[i];
      // get coordinates of the centre of the vertex
      int x = convertx(p.getX(),  wd);
      int y = converty(p.getY(), ht);

      if ((x >= xmin) && (x <= xmax) &&  (y >= ymin) && (y <= ymax))
      {
        return p;
      }
    }
    return null;
  }

  public void paintElem(Graphics g, Point3D p, int wd, int ht) {
    int x = convertx(p.getX(), wd);
    int y = converty(p.getY(), ht);
    g.setColor(this.getBackground());
    g.fillOval(x - pointRadius, y - pointRadius, pointDiameter, pointDiameter);
    Color color = (Color)filledVertices.get(p.getLabel());
    if (color != null) {
      g.setColor(color);
      g.fillOval(x - pointRadius,y - pointRadius,pointDiameter,pointDiameter);
    }
    boolean thick = false;
    Vector drawColor = (Vector)drawnVertices.get(p.getLabel());
    if (drawColor != null) {
      thick = ((Boolean)drawColor.elementAt(1)).booleanValue();
      g.setColor((Color)drawColor.elementAt(0));
    }
    else {
      g.setColor(this.getForeground());
    }
    g.drawOval(x - pointRadius,y - pointRadius,pointDiameter,pointDiameter);
    if (thick) {
      g.drawOval(x - pointRadius - 1, y - pointRadius - 1,
                            pointDiameter + 2, pointDiameter + 2);
      g.drawOval(x - pointRadius - 1, y - pointRadius - 1,
                            pointDiameter + 3, pointDiameter + 3);
      g.drawOval(x - pointRadius - 2, y - pointRadius - 2,
                            pointDiameter + 3, pointDiameter + 3);
    }
  }

  public void eraseElem(Graphics g, Point3D p, int wd, int ht) {
    int x = convertx(p.getX(), wd);
    int y = converty(p.getY(), ht);
    g.setColor(this.getBackground());
    g.fillOval(x - pointRadius, y - pointRadius, pointDiameter, pointDiameter);
  }

  int convertx(float x, int wd) {
    return (int)Math.round(x * wd);
  } 

  int converty(float y, int ht) {
    return (int)Math.round((1 - y) * ht);
  } 

  void rotateOnce () {
    rotationAngle += rotAngleDelta;
    if (rotationAngle > 2*Math.PI) rotationAngle -= 2*Math.PI;
    diagram.setRotationAngle(rotationAngle);
    diagram.project2d(rotationAngle);
    repaint();
  }

  void rotateLeft () {
    rotationAngle -= rotAngleDelta;
    if (rotationAngle < -2*Math.PI) rotationAngle += 2*Math.PI;
    diagram.setRotationAngle(rotationAngle);
    diagram.project2d(rotationAngle);
    repaint();
  }

  void rotateStart () {
    rotating = true;
    if (rotateThread != null && rotateThread.isAlive()) rotateThread.stop();
    rotateThread = new RotateThread(this);
    rotateThread.start();
  }

  void rotateStop () {
    rotating = false;
    if(rotateThread != null && rotateThread.isAlive()) {
      rotateThread.stop();
      rotateThread = null;
    }
    diagram.setRotationAngle(rotationAngle);
  }


/*
  void improveOnce () {
    if (improvementCount < phase1) {
      diagram.update(att_I, rep_I);
    } else if (phase1 <= improvementCount && improvementCount < phase2) {
      diagram.update(att_II, rep_II);
    } else if (phase2 <= improvementCount && improvementCount < phase3) {
      diagram.update(att_III, rep_III);
    } else {
      diagram.update(att_user, rep_user);
    }

    countDown.decrementCount();
    countDown.repaint();
    improvementCount++;
    float scale = diagram.center();
    diagram.normalizeCoords(scale);
    diagram.project2d(rotationAngle);
    repaint();
  }
*/

  void improveOnce () {
    improveMany(1, true);
  }

  void improveMany (int k, boolean repaint) {
    for (int i = 0; i < k; i++) {
      if (improvementCount < phase1) {
        diagram.update(att_I, rep_I);
      } else if (phase1 <= improvementCount && improvementCount < phase2) {
        diagram.update(att_II, rep_II);
      } else if (phase2 <= improvementCount && improvementCount < phase3) {
        diagram.update(att_III, rep_III);
      } else {
        diagram.update(att_user, rep_user);
      }
      improvementCount++;
      countDown.decrementCount();
    }
    if (repaint) {
      if  (latDrawWin.showSteps) countDown.repaint();
      float scale = diagram.center();
      diagram.normalizeCoords(scale);
      diagram.project2d(rotationAngle);
      repaint();
    }
  }

}

class Bottom extends Panel {

  public Bottom(LatDrawPanel dp,Controls c, MessageArea ma, CountDown cd, 
                        GetURLPanel urlPanel, IntervalPanel intervalPanel) {
    setLayout(new BorderLayout());
    add("North", new ForceControls(dp));
    add("South", new ControlsAndMessage(dp,c,ma,cd,urlPanel, intervalPanel));
    this.repaint();
  }
}

class ControlsAndMessage extends Panel {

  MessageArea ma;
  GetURLPanel urlPanel;
  IntervalPanel intervalPanel;
  Controls c;
  CountDown cd;
  LatDrawPanel dp;
  int mode;
  final static int MESSAGE_MODE = 0;
  final static int URL_MODE = 1;
  final static int INTERVAL_MODE = 2;


  public ControlsAndMessage (LatDrawPanel dp,Controls c, MessageArea ma, 
              CountDown cd, GetURLPanel urlPanel, IntervalPanel intPanel) {
    this.ma = ma;
    this.urlPanel = urlPanel;
    this.intervalPanel = intPanel;
    this.c = c;
    this.cd = cd;
    this.dp = dp;
    dp.controlsAndMessage = this;
    setLayout(new BorderLayout());
    add("South",ma);
    //add("South",urlPanel);
    add("Center",cd);
    add("North",c);
    this.repaint();
  }

  public int getMode() { return mode; }

  void setURLMode () {
    //remove(ma);
    //add(urlPanel);
    dp.filledVertices.clear();
    intervalPanel.stopListening();
    dp.latChooser.disableChoosing();
    dp.latDrawWin.setDisallowOutsideAction(true);
    mode = URL_MODE;
    removeAll();
    setLayout(new BorderLayout());
    add("North",c);
    add("Center",cd);
    add("South",urlPanel);
    validate();    // tells the layout manager to start over
    //urlPanel.repaint();
    this.repaint();
    urlPanel.messageArea.repaint();
  }

  void setIntervalMode () {
    //remove(ma);
    //add(urlPanel);
    mode = INTERVAL_MODE;
    dp.highlightedEdges.clear();
    dp.drawnVertices.clear();
    dp.filledVertices.clear();
    dp.latChooser.disableChoosing();
    dp.latDrawWin.setDisallowOutsideAction(true);
    intervalPanel.messageArea.setMessage(intervalPanel.intervalMessage);
    intervalPanel.messageArea.repaint();
    intervalPanel.startListening();
    intervalPanel.one = dp.diagram.getOrderedSet().one().label();
    intervalPanel.zero = dp.diagram.getOrderedSet().zero().label();
    intervalPanel.top = intervalPanel.one;
    intervalPanel.bottom = intervalPanel.zero;
    intervalPanel.colorTopBottom();
    removeAll();
    setLayout(new BorderLayout());
    add("North",c);
    add("Center",cd);
    add("South",intervalPanel);
    validate();    // tells the layout manager to start over
    intervalPanel.messageArea.repaint();
  }

  void setMessageMode () {
    mode = MESSAGE_MODE;
    intervalPanel.stopListening();
    dp.latChooser.enableChoosing();
    dp.latDrawWin.setDisallowOutsideAction(false);
    removeAll();
    setLayout(new BorderLayout());
    add("North",c);
    add("Center",cd);
    add("South",ma);
    validate();    // tells the layout manager to start over
    ma.repaint();
  }

}

class CountDown extends Canvas {

  int count;
  int max;
  boolean show = true;

  public CountDown(boolean show) {
    this();
    this.show = show;
  }


  public CountDown () {
    setForeground(Color.black);
    count = 0;
    max = 1;
  }

  public void setMax (int m) {
    max = m;
  }

  public void setCount (int c) {
    count = c;
  }

  public void decrementCount () {
    count--;
  }

  public void resetParameters (int c) {
    count = c;
    max = c;
    repaint();
  }

  public void resetParameters () {
    count = 0;
    max = 1;
    repaint();
  }

  public void paint(Graphics g) {
    if (show) {
      Rectangle r = this.bounds();
      int wd = r.width;
      int ht = r.height;
      int pos = wd*(max - count)/max;
      if (count <= 0) {
        g.setColor(Color.lightGray);
        //g.setColor(Color.cyan);
        g.fillRect(0, 0, wd, ht);
      } else {
        g.setColor(Color.red);
        g.fillRect(pos, 0, wd, ht);
        g.setColor(Color.green);
        g.fillRect(0, 0, pos, ht);
      }
    }
  }

}

  // This used to extend Panel but then the resize (called in init) didn't work.
 
class MessageArea extends Canvas {

  static int bigFontSize = 24;
  static int smallFontSize = 16;
  static int tinyFontSize = 12;
  //static Font f = new Font("TimesRoman",Font.ITALIC+Font.BOLD,fontSize);
  static Font smallFont = new Font("Helvetica",Font.PLAIN,smallFontSize);
  static Font tinyFont = new Font("Helvetica",Font.PLAIN,tinyFontSize);
  //static Font bigFont = new Font("Helvetica",Font.BOLD, bigFontSize);
  static Font bigFont = new Font("TimesRoman",Font.BOLD, bigFontSize);
  FontMetrics big_fm; 
  FontMetrics tiny_fm; 
  FontMetrics small_fm;

  String[][] messages;
  int currentMessage;

  public MessageArea(String[][] m) {
    big_fm = getFontMetrics(bigFont);
    small_fm = getFontMetrics(smallFont);
    tiny_fm = getFontMetrics(tinyFont);
    messages = m;
    currentMessage = 0;
    setBackground(Color.lightGray);
  }

  public void setMessage(String[][] m) {
    messages = m;
  }

  public void nextMessage() {
    if (currentMessage < messages.length - 1) currentMessage++;
  }

  public void resetCurrentMessage(Diagram d) {
    // This should check what phase we are in but now we are doing
    // all 3 phases at once so it doesn't matter.
    if (d.getImprovementCount() > 0) {
      currentMessage = messages.length - 1;
    }
    else {
      currentMessage = 0;
    }
    repaint();
  }
 

  public void resetCurrentMessage() {
    currentMessage = 0;
    repaint();
  }

  public void paint(Graphics g) {
    if (currentMessage < messages.length) {
      Rectangle r = this.bounds();
      g.setColor(this.getBackground());
      g.fillRect(r.x, r.y, r.width, r.height);
      g.setColor(Color.blue);
      g.drawRect(1, 1, this.bounds().width - 3, this.bounds().height - 3);
      g.setColor(this.getForeground());
      g.setFont(bigFont);
      int y_pos = bigFontSize + 4;
      drawCenteredString(messages[currentMessage][0],g,big_fm,y_pos);

      g.setFont(smallFont);
      int y_delta = smallFontSize + 4;
      for(int i = 1; i < messages[currentMessage].length - 1; i++) {
        y_pos += y_delta;
        //g.drawString(messages[currentMessage][i],10,y_pos);
        drawCenteredString(messages[currentMessage][i],g,small_fm,y_pos);
      }
      // Do the last message line in tiny
      g.setColor(this.getForeground());
      g.setFont(tinyFont);
      y_pos += smallFontSize + 4;
      drawCenteredString(
        messages[currentMessage][messages[currentMessage].length-1],
        g,tiny_fm,y_pos);


    }
  }

  void drawCenteredString(String s, Graphics g, FontMetrics fm, int y) {
    int x = (this.size().width - fm.stringWidth(s))/2;
    g.drawString(s, x, y);
  }
}

class GetURLPanel extends Panel {

  static String [][] urlMessage = {{"Input Your Own Lattice", 
                 "Type in an URL with the lattice. See the directions below"}};
  LatDrawPanel target;
  MessageArea messageArea;
  Panel URL;
  Button okButton;
  Button cancelButton;
  Button clearButton;
  TextField textField;

  public GetURLPanel(LatDrawPanel target) {
    this.target = target;
    messageArea = new MessageArea(urlMessage);
    messageArea.resize(target.bounds().width,61);
    textField = new TextField("http://",35);
    okButton = new Button("OK");
    cancelButton = new Button("Cancel");
    clearButton = new Button("Clear");
    Panel URLPanel = new Panel();
    URLPanel.setLayout(new FlowLayout()); 
    URLPanel.add(textField);
    URLPanel.add(okButton);
    URLPanel.add(cancelButton);
    URLPanel.add(clearButton);
    setLayout(new BorderLayout());
    //add("NORTH", okButton);
    add("Center", messageArea);
    add("South", URLPanel);
  }

  public boolean action(Event e, Object arg) {
    if (e.target == cancelButton) {
      target.controlsAndMessage.setMessageMode();
      return true;
    }
    if (e.target == clearButton) {
      textField.setText("");
      return true;
    }
    if (e.target == okButton || e.target == textField) {
      InputLattice in;
      Diagram diagram = null;
      try {
        in = new InputLattice(textField.getText(), InputLattice.URL);
      } catch (FileNotFoundException ex) {
        handleError(ex.getMessage() + " file not found.");
        return true;
      } catch (IOException ex) {
        handleError(ex.getMessage() + 
                 ": bad URL, or file not found, or empty file");
        return true;
      }
      try {
        diagram = new Diagram(in);
      } catch (NonOrderedSetException ex) {
        if (NonOrderedSetException.emptyError.equals(ex.getMessage())) {
          handleError(ex.getMessage() + " Or the file did not exist.");
        } else {
          handleError(ex.getMessage());
        }
        return true;
      }
      if (diagram != null) {
        if (target.allLats.size() == target.latChooser.max_lats) {
          shiftVector(target.allLats);
          target.allLats.setElementAt(diagram, 
                                target.latChooser.max_lats - 1);
        } else {
          target.allLats.addElement(diagram);
          target.latChooser.addChoice();
        } 
        target.setup(diagram, true);
        target.countDown.resetParameters();
        target.controls.improveButtonDark();
        target.controlsAndMessage.ma.resetCurrentMessage(target.diagram);
        target.controlsAndMessage.setMessageMode();
        target.repaint();
      }
      return true;
    }
    return false;
  }

  public void handleError(String err) {
    if (target.latDrawWin.isStandalone) {
      System.out.println(err);
    } else {
      target.latDrawWin.showStatus(err);
    }
  }

  /**
    * pop of the first element of a vector, move everything up one.
    */
  public void shiftVector(Vector v) {
    int k = v.size() - 1;
    for(int i = 0; i < k; i++) {
      v.setElementAt(v.elementAt(i+1),i);
    }
  }

}
  
class IntervalPanel extends Panel {

  static final String [][] intervalMessage = {{
                 "Form Interval: left click bottom; right click top", 
                 "The bottom is green, the top is blue.",
                 "The interval must be proper and nontrivial."
                 //"For a filter just left click the bottom. Same for ideal."
               }};

/*
  static final String [][] notProperMessage = {{
           "You must choose a proper, nontrivial interval",
           "Choose again: left click bottom; right click top." }};

  static final String [][] notIntervalMessage = {{
           "The green must be below the blue",
           "Choose again: left click bottom; right click top." }};
*/


  LatDrawPanel target;
  MessageArea messageArea;
  Panel interval;
  Button okButton;
  Button cancelButton;
  Button resetButton;

  String top, bottom;
  String one, zero;


  PropertyChangeListener eltClick = new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent e) {
          String prop = e.getPropertyName();
          if ("VertexClicked".equals(prop)) {
            bottom = (String)e.getNewValue();
            colorTopBottom();
            target.repaint();
          }
          if ("VertexRightClicked".equals(prop)) {
            top = (String)e.getNewValue();
            colorTopBottom();
            target.repaint();
          }
          if ((!top.equals(bottom)) 
               && (! (bottom.equals(zero) && top.equals(one)))
               && target.diagram.getOrderedSet().leq(
                     target.diagram.getOrderedSet().getElement(bottom),
                     target.diagram.getOrderedSet().getElement(top))) {
            okButton.enable();
          }
          else {
            okButton.disable();
          }
        }
      };

  public void startListening() { 
    target.listeners.addPropertyChangeListener(eltClick);
  }

  public void stopListening() { 
    target.listeners.removePropertyChangeListener(eltClick);
  }

  public void allowOutsideAction() { 
    target.latDrawWin.setDisallowOutsideAction(false);
  }

  public void colorTopBottom() {
    target.filledVertices.clear();
    target.filledVertices.put(bottom, Color.green);
    target.filledVertices.put(top, Color.blue);
  }
  

  public IntervalPanel(LatDrawPanel target) {
    this.target = target;
    //startListening();
    one = target.diagram.poset.one().label();
    top = one;
    zero = target.diagram.poset.zero().label();
    bottom = zero;
    messageArea = new MessageArea(intervalMessage);
    messageArea.resize(target.bounds().width, 86);
    okButton = new Button("OK");
    okButton.disable();
    cancelButton = new Button("Cancel");
    resetButton = new Button("Reset");
    Panel IntervalPanel = new Panel();
    IntervalPanel.setLayout(new FlowLayout()); 
    IntervalPanel.add(okButton);
    IntervalPanel.add(cancelButton);
    IntervalPanel.add(resetButton);
    setLayout(new BorderLayout());
    //add("NORTH", okButton);
    add("Center", messageArea);
    add("South", IntervalPanel);
  }

  public boolean action(Event e, Object arg) {
    if (e.target == cancelButton) {
      //if (bottom != null) target.filledVertices.remove(bottom);
      //if (top != null) target.filledVertices.remove(top);
      okButton.disable();
      target.filledVertices.clear();
      target.controlsAndMessage.setMessageMode();
      stopListening();
      allowOutsideAction();
      target.latChooser.enableChoosing();
      target.repaint();
      return true;
    }
    if (e.target == resetButton) {
      top = one;
      bottom = zero;
      colorTopBottom();
      okButton.disable();
      target.repaint();
      return true;
    }
    if (e.target == okButton) {
/*
      if (top.equals(one) && bottom.equals(zero)) {
        messageArea.setMessage(notProperMessage);
        messageArea.repaint();
        return true;
      }
      if (top.equals(bottom)) {
        messageArea.setMessage(notProperMessage);
        messageArea.repaint();
        return true;
      }
*/
      Diagram diagram = null;
      try {
        diagram = target.diagram.interval(top, bottom);
      } catch (NonOrderedSetException ex) {
        ex.printStackTrace();
        //messageArea.setMessage(notIntervalMessage);
        //messageArea.repaint();
        return true;
      }
      if (diagram != null) {
        //if (bottom == null) bottom = diagram.getOrderedSet().zero().label();
        //if (top == null) top = diagram.getOrderedSet().one().label();
        //String pname = makeName(target.diagram.getName(), bottom, top);
        //if (pname == null) pname = "";
        //diagram.setName(pname + "[" + bottom + "," + top + "]");
        diagram.setName(makeName(target.diagram.getName(), bottom, top));
        diagram.setInterval(true);
        diagram.setEdgeColors(target.diagram.getEdgeColors());
        // should also clear all vertex color stuff in target. Skip for now.
        // (or maybe target's setup should do this.)
        if (target.allLats.size() == target.latChooser.max_lats) {
          shiftVector(target.allLats);
          target.allLats.setElementAt(diagram, 
                                target.latChooser.max_lats - 1);
        } else {
          target.allLats.addElement(diagram);
          target.latChooser.addChoice();
        } 
        stopListening();
        okButton.disable();
        allowOutsideAction();
        target.setup(diagram, true);
        target.latChooser.enableChoosing();
        target.listeners.removePropertyChangeListener(eltClick);
        target.countDown.resetParameters();
        target.controls.improveButtonDark();
        target.controlsAndMessage.ma.resetCurrentMessage();
        target.controlsAndMessage.setMessageMode();
        target.filledVertices.clear();
        target.repaint();
      }
      return true;
    }
    return false;
  }

  String makeName(String oname, String bottom, String top) {
    if (oname == null) oname = "";
    int index = oname.indexOf("[");
    if (index != -1) {
      oname = oname.substring(0, index);
    }
    else {
      if (oname.length() > 6) oname = oname.substring(0, 4) + "..";
    }
    return oname + "[" + bottom + "," + top + "]";
  }

  public void handleError(String err) {
    if (target.latDrawWin.isStandalone) {
      System.out.println(err);
    } else {
      target.latDrawWin.showStatus(err);
    }
  }

  /**
    * pop of the first element of a vector, move everything up one.
    */
  public void shiftVector(Vector v) {
    int k = v.size() - 1;
    for(int i = 0; i < k; i++) {
      v.setElementAt(v.elementAt(i+1),i);
    }
  }

}
  
    

class Controls extends Panel {
  LatDrawPanel target;
  MessageArea messageArea;
  Button improveButton;
  Button rotateButton;
  Button rotateRightButton;
  Button rotateLeftButton;
  Button inputButton;
  Button intervalButton;
  int stepSize = 1;

  public Controls(LatDrawPanel target, MessageArea ma) {
    this.target = target;
    if (target.latDrawWin.showSteps) stepSize = target.latDrawWin.stepSize;
    messageArea = ma;
    setLayout(new FlowLayout());
    setBackground(Color.lightGray);
    improveButton = new Button("Improve");
    add(improveButton);
    rotateButton = new Button("Rotate");
    add(rotateButton);
    rotateRightButton = new Button("Rotate >>");
    add(rotateRightButton);
    rotateLeftButton = new Button("Rotate <<");
    add(rotateLeftButton);
    if (target.latDrawWin.includeInput) {
      inputButton = new Button("Input");
      add(inputButton);
    }
    intervalButton = new Button("Interval");
    add(intervalButton);
    target.repaint();
  }

    // See not in LatDrawWin.java for the reason for deleting 
    // this repaint.

    public void improveButtonDark () {
      // improveButton.setForeground(Color.black);
      // //improveButton.repaint();
      // repaint();
      target.improving = false;
      improveButton.enable();
    }

    public void improveButtonFaded () {
      // improveButton.setForeground(Color.gray);
      // //improveButton.repaint();
      // repaint();
      target.improving = true;
      improveButton.disable();
    }

    public void setButtonLabels () {
      rotateButton.setLabel("Rotate");
      //improveButton.setLabel("Next");
      improveButtonDark();
      repaint();
    }
      

    public void paint(Graphics g) {
      Rectangle r = bounds();

      g.setColor(Color.lightGray);
      g.draw3DRect(0, 0, r.width, r.height, false);
      g.setColor(Color.green);
      g.drawRect(1,1,r.width - 3, r.height - 3);
    }

    public boolean action(Event e, Object arg) {
      if (e.target instanceof Checkbox) {
        //target.setForeground(((Component)e.target).getBackground());
      } else if (e.target instanceof Choice) {
        String choice = (String)arg;

      } else if (e.target == rotateButton) {
        if (target.rotating) {
          rotateButton.setLabel("Rotate");
          repaint();
          target.rotateStop();
        } else {
          rotateButton.setLabel("Stop");
          repaint();
          target.rotateStart();
        } 
        target.repaint();
        target.postEvent(new Event(target,Event.MOUSE_MOVE,null));
        // target.processEvent(new AWTEvent(target,));
      } else if (e.target == improveButton && !target.improving) {
        // target.improving = true;
        messageArea.nextMessage();
        if (target.latDrawWin.showSteps) messageArea.repaint();
        int c = target.improvementCount;
        int phase1 = target.phase1;
        int phase2 = target.phase2;
        int phase3 = target.phase3;
        if (c < phase1) {
          target.forceControls.setForces(target.att_I,target.rep_I);
          target.countDown.resetParameters(phase1 );
        } else if (c < phase2) {
          target.forceControls.setForces(target.att_II,target.rep_II);
          target.countDown.resetParameters(phase2 - phase1);
          } else if (c < phase3) {
          target.forceControls.setForces(target.att_III,target.rep_III);
          target.countDown.resetParameters(phase3 - phase2);
        } else {
          target.att_user = target.forceControls.getAttraction();
          target.rep_user = target.forceControls.getRepulsion();
          target.countDown.resetParameters(40 * stepSize);
        }

        improveButtonFaded();

        Thread fixIt = new ImproveThread(target);
        target.improveThread = fixIt;
        fixIt.start();

        target.repaint();
      } else if (e.target == rotateRightButton) {
        target.rotateOnce();
      } else if (e.target == rotateLeftButton) {
        target.rotateLeft();
      } else if (e.target == inputButton) {
        if (target.controlsAndMessage.getMode() 
                      != target.controlsAndMessage.URL_MODE) {
          target.controlsAndMessage.setURLMode();
        }

      } else if (e.target == intervalButton) {
        if (target.controlsAndMessage.getMode() 
                      != target.controlsAndMessage.INTERVAL_MODE) {
          target.controlsAndMessage.setIntervalMode();
        }
      }

// {
// } else if (e.target == infoButton) {
// }
// Frame fr = FindParentFrame.getFrame(this);
// InfoDialog info = new InfoDialog(fr, "Testing", "This is a test.");
// info.show();


      target.repaint();
      return true;
    }

}

class ForceControls extends Panel {
  LatDrawPanel target;
  //Scrollbar attBar;
  //Scrollbar repBar;
  AttRepSlider attBar, repBar;
  Label attLabel;
  Label repLabel;
  int att;      // the real attraction * 300
  int rep;      // the real repulsion * 30
  
  public ForceControls (LatDrawPanel dp) {
    target = dp;
    dp.forceControls = this;
    setLayout(new FlowLayout());
    //attBar = new Scrollbar(Scrollbar.HORIZONTAL,30,120,10,120);
    //repBar = new Scrollbar(Scrollbar.HORIZONTAL,30,120,10,120);
    attBar = new AttRepSlider("attraction");
    repBar = new AttRepSlider("replusion");
    //attLabel = new Label("Attraction 30  ");
    //repLabel = new Label("Repulsion 30  ");
    attLabel = new Label("Attraction: ");
    repLabel = new Label("Repulsion: ");
    add(attLabel);
    add(attBar);
    add(repLabel);
    add(repBar);
  }

  public void setForces(float attraction, float repulsion) {
    setForces(attraction, repulsion, true);
  }

  public void setForces(float attraction, float repulsion, boolean show) {
    att = Math.round((attraction/target.attFactor)*(float)30.0);
    rep = Math.round((repulsion/target.repFactor)*(float)30.0);
    attBar.setValue(att);
    repBar.setValue(rep);
    //attLabel.setText("Attraction " + att + "  ");
    //repLabel.setText("Repulsion " + rep + "  ");
    // some how it repainted anyway. later
    if (show) repaint();
  }

  public float getAttraction() {
    return (attBar.getValue() / (float)30.0) * target.attFactor;
  }

  public float getRepulsion() {
    return (repBar.getValue() / (float)30.0) * target.repFactor;
  }

/*
  public boolean handleEvent(Event e) {
    if (e.target == attBar) {
      int v = ((Scrollbar)e.target).getValue();
      att = v;
      attLabel.setText("Attraction " + v);
    } else if (e.target == repBar) {
      int v = ((Scrollbar)e.target).getValue();
      rep = v;
      repLabel.setText("Repulsion " + v);
    }
    return true;
  }
*/
}

class AttRepSlider extends Slider {
  ForceControls parent;
  String name;    // will be either attraction or replusion

  /**
   * Construct a slide of this type
   * @param name Either attraction or replusion.
   */
  public AttRepSlider (String name) {
    super();
    this.name = name;
    SetBarColor(Color.white);
    SetMinimum(10);
    SetMaximum(120);
    SetValue(30);
  }


/*
  public void Release () {
    if (name == "attraction") parent.att = GetValue();
    if (name == "replusion")  parent.rep = GetValue();
  }
*/
  
}

class LatChooser extends Panel {
    LatDrawPanel target;
    Controls controls;
    MessageArea messageArea;
    CountDown countDown;
    CheckboxGroup grp;
    Vector cbxvec;
    int fixed_choices; 
    int user_choices = 0; 
    boolean choosingEnabled = true;
    // this will hold the chbox for the current diagram when we are in
    // interval choosing mode.
    Checkbox checkBox;
    static final int max_lats = 9;
    

    public LatChooser(LatDrawPanel dp,Controls c, MessageArea ma) {
      target = dp;
      cbxvec = new Vector();
      fixed_choices = target.latstr.length;
      controls = c;
      messageArea = ma;
      countDown = dp.countDown;
      setBackground(Color.lightGray);
      setLayout(new GridLayout(fixed_choices + 1,1));
      grp = new CheckboxGroup();
      for (int i = 0; i < target.allLats.size();i++)
      {
        Diagram tmp=(Diagram) target.allLats.elementAt(i);
        cbxvec.addElement(new Checkbox(tmp.getName(), grp,false));
        add((Checkbox) cbxvec.elementAt(i));
      }
    }

    public void enableChoosing() { choosingEnabled = true; }

    public void disableChoosing() { 
      choosingEnabled = false; 
      checkBox = grp.getSelectedCheckbox();
    }

    public void addChoice() {
      invalidate();
      String tmpname = 
        ((Diagram) target.allLats.lastElement()).getName();
      if (tmpname == null) {
        tmpname = "User" + target.allLats.size();
      }
      // rsf new
      cbxvec.addElement(new Checkbox(tmpname,grp,true));
      //add(new Checkbox(tmpname,grp,true));
      add((Checkbox)cbxvec.lastElement());
      setLayout(new GridLayout(target.allLats.size(),1));
      validate();
      target.latDrawWin.validate();
      target.repaint();
    }
      
    public void paint(Graphics g) {
      Rectangle r = bounds();
      g.setColor(Color.lightGray);
      g.draw3DRect(0, 0, r.width, r.height, true);
      //g.setColor(Color.yellow);
      //g.drawRect(1,1,r.width - 3, r.height - 3);
      //g.drawRect(0,0,r.width, r.height);
    }

// also need to repaint the buttons and messages

    public boolean action(Event e, Object arg) {
      if (!choosingEnabled) {
        grp.setSelectedCheckbox(checkBox);
      }
      else if (e.target instanceof Checkbox) {
        String lat = ((Checkbox)e.target).getLabel();
        int k = 0;
        //AS: map checkbox back to its index...
        for (int i=0; i<cbxvec.size();i++) {
          if (((Checkbox)e.target) == cbxvec.elementAt(i)) {
            k = i;
          }
        }
        target.setup((Diagram)target.allLats.elementAt(k));
        target.repaint();
        controls.improveButtonDark();
        controls.setButtonLabels();
        messageArea.resetCurrentMessage(target.diagram);
        countDown.resetParameters();
      }
      return true;
    }

}

class ImproveThread extends Thread {

  LatDrawPanel dp;
  MessageArea messageArea; // rsf new
  private int phase1, phase2, phase3;
  boolean showSteps;
  int stepSize;
  //static private int improveDelay = 150;
  static private int improveDelay = 50;

  public ImproveThread(LatDrawPanel dp) {
    this.dp = dp;
    phase1 = dp.phase1;
    phase2 = dp.phase2;
    phase3 = dp.phase3;
    messageArea = dp.latDrawWin.messageArea; // rsf new
    showSteps = dp.latDrawWin.showSteps;
    stepSize = dp.latDrawWin.stepSize;
  }


  public void run () {
    if (dp.improvementCount < phase3) {
      while (dp.improvementCount < phase1) {
        //dp.improveOnce();
        if (showSteps) {
          dp.improveMany(stepSize, true);
        } 
        else {
          dp.improveMany(phase1, false);
        }
        try {
          Thread.sleep(improveDelay);
        } catch (InterruptedException e) { }
      }
      messageArea.nextMessage();
      if (showSteps) messageArea.repaint();
      dp.forceControls.setForces(dp.att_II,dp.rep_II, showSteps);
      dp.countDown.resetParameters(phase2 - phase1);
      while (dp.improvementCount < phase2) {
        //dp.improveOnce();
        if (showSteps) {
          dp.improveMany(stepSize, true);
        } 
        else {
          dp.improveMany(phase2 - phase1, false);
        }
        try {
          Thread.sleep(improveDelay);
        } catch (InterruptedException e) { }
      }
      messageArea.nextMessage();
      if (showSteps) messageArea.repaint();
      dp.forceControls.setForces(dp.att_III,dp.rep_III, showSteps);
      dp.countDown.resetParameters(phase3 - phase2);
      while (dp.improvementCount < phase3) {
        //dp.improveOnce();
        if (showSteps) {
          dp.improveMany(stepSize, true);
        } 
        else {
          dp.improveMany(phase3 - phase2, true);
        }
        try {
          Thread.sleep(improveDelay);
        } catch (InterruptedException e) { }
      }
      messageArea.nextMessage();
      messageArea.repaint();
      dp.diagram.setImprovementCount(dp.improvementCount);
    } else {
      if (showSteps) {
        for(int k = 0; k < 40; k++) {
          //dp.improveOnce();
          dp.improveMany(stepSize, true);
          try {
            Thread.sleep(improveDelay);
          } catch (InterruptedException e) { }
        }
      }
      else {
        dp.improveMany(40, true);
      }
    } 
    dp.controls.improveButtonDark();
  }

  public void runOrig () {
    if (dp.improvementCount < phase1) {
      while (dp.improvementCount < phase1) {
        dp.improveOnce();
        try {
          Thread.sleep(improveDelay);
        } catch (InterruptedException e) { }
      }
    } else if (dp.improvementCount < phase2) {
      while (dp.improvementCount < phase2) {
        dp.improveOnce();
        try {
          Thread.sleep(improveDelay);
        } catch (InterruptedException e) { }
      }
    } else if (dp.improvementCount < phase3) {
      while (dp.improvementCount < phase3) {
        dp.improveOnce();
        try {
          Thread.sleep(improveDelay);
        } catch (InterruptedException e) { }
      }
    } else {
      for(int k = 0; k < 40; k++) {
        dp.improveOnce();
        try {
          Thread.sleep(improveDelay);
        } catch (InterruptedException e) { }
      }
    } 
    dp.controls.improveButtonDark();
  }


}
  
class RotateThread extends Thread {

  static private int rotateDelay = 150;

  LatDrawPanel dp;

  public RotateThread(LatDrawPanel dp) {
    this.dp = dp;
  }

  public void run() {
    while (true) {
      dp.rotateOnce();
      try {
        Thread.sleep(rotateDelay);
      } catch (InterruptedException e) { }
    }
  }

}

class LatFrame extends Frame {

  public LatFrame (String str) {
    super(str);
  }
  
/*
  public boolean action(Event e, Object arg) {
    if (e.target instanceof MenuItem) {
      String label = (String)arg;
      if (label.equals("Quit")) {
        System.exit(0);
        return true;
      }
    }
    return false;
  }
*/

  public boolean handleEvent(Event e) {
    switch (e.id) {
      case Event.WINDOW_DESTROY:
        System.exit(0);
        return true;
      case Event.ACTION_EVENT:
        if (e.target instanceof MenuItem) {
          if (((String)e.arg).equals("Quit")) {
            System.exit(0);
            return true;
          } 
        }
      default:
        return false;
    }
  }

}

