package javamath.hasse.latdraw;

/* Point3D.java  (c) Ralph Freese 97/01/07

 * I am adding code which saves the last force calculation and takes the 
 * dot product with the new force. If this is positive the movement is 
 * increased but if it is negative it is decreased. This should prevent
 * wild oscillations. The formula I am using is to multiply v_new
 * (the new force) by
 *
 *                v_new \cdot v_old  
 *        1 + 0.5 -----------------
 *                |v_new| * |v_old|
 * 
 * 0.5 is the constant named factor; there is nothing scientific about it;
 * it just seems to work well.
 * <p> 
 * @version $Id: Point3D.java,v 1.4 2001/06/05 00:14:27 ralphfreese Exp $
 */
 

class Point2D {
  float x;
  float y;

  public Point2D () {
    x = (float)0.0;
    y = (float)0.0;
  }

/**
 * The usual dot product.
 * @param pt The other point (vector)
 */
  public float dotProduct(Point2D pt) {
    return (x * pt.x + y * pt.y);
  }

/**
 * The Euclidean length.
 */
  public float length() {
    return (float)java.lang.Math.sqrt(x * x + y * y);
  }

/**
 * Correlation between two points (vectors).
 * @param pt The other point (vector)
 */
  public float correlation(Point2D pt) {
    float len1 = this.length();
    float len2 = pt.length();
    if (len1 == (float)0.0 || len2 == (float)0.0) {
      return((float)0.0);
    } else {
      return (dotProduct(pt) / (len1 * len2));
    }
  }

}

public class Point3D extends Point2D {
  int z;
  private Point2D currentForce;
  private Point2D previousForce;
  private static float factor = (float)0.5;
  Point2D proj2d;
  float normalizedX;  // These will be x, y, z scaled so that they are  
  float normalizedY;  // between 0 and 1.
  float normalizedZ;
  String label;

  public String getLabel()
  {
    return label;
  }

  public Point3D (String n) {
    currentForce = new Point2D();
    previousForce = null;
    proj2d = new Point2D();
    label = new String(n);
  }

  public Point2D getProjection() {
    return proj2d;
  }

  public float getX () {
    return proj2d.x;
  }

  public float getY () {
    return proj2d.y;
  }

  public void update () {
    float correction = (float)1.0;
    if (previousForce != null) {
      correction = (float)1.0 
                     + factor * currentForce.correlation(previousForce);
    } else {
      previousForce = new Point2D();
    }
    x += correction * currentForce.x;
    y += correction * currentForce.y;
    previousForce.x = currentForce.x;
    previousForce.y = currentForce.y;
    currentForce.x = (float)0.0;
    currentForce.y = (float)0.0;
  }

  public void adjustForce (float dx, float dy) {
    currentForce.x += dx;
    currentForce.y += dy;
  }

  protected void setNormalizedCoords (float scaleFactor) {
    normalizedX = x/scaleFactor;
    normalizedY = y/scaleFactor;
    normalizedZ = z/scaleFactor;
  }

  public void project2d(float angle) {
    proj2d.x = (float)Math.cos(angle) * normalizedX 
               + (float)Math.sin(angle) * normalizedY;
    proj2d.y = normalizedZ;
  }

    

  public String toString() {
    return "("+getLabel()+":" + x + ", " + y + ", " + z + ")" 
                 + "[" + proj2d.x + ", " + proj2d.y + "]";
  }
}

