package uacalc.alg;

import uacalc.ex.TooManyArgumentsException;

public class OpArguments {
//This class represents the list of arguments of an operation.
//The argument list can be empty for nullary operations.

    public OpArguments(int NumArgs) throws TooManyArgumentsException {
//It may be a very bad idea to throw this exception here!!!

	if(NumArgs < 0 || NumArgs >= Operation.ARITY_LIMIT) {
	    throw new TooManyArgumentsException(
		"Too many arguments " + NumArgs +" (should be in [" +
		0 + "," +  (Operation.ARITY_LIMIT-1) + "]).");
	}
	m_NumArgs = NumArgs;
	m_args = new int[NumArgs];
	int i;
	for(i=0; i< NumArgs; i++) {
	    m_args[i]=0;
	}
    }

    public int getLength() {
	return m_NumArgs;
    }

    public int getValue(int i) {
	return m_args[i];
    }

    public int[] getValues() {
	return m_args;
    }

    public void setValue(int i, int value) {
	m_args[i] = value;
    }

    public void setValues(int value[]) {
	//DOES NOT CHECK VALIDITY!
	m_args = value;
	m_NumArgs = value.length;
    }

    public int getNumValue(int size) {
	//returns the "value" of args as a base "size" number.
	// Uses the Horner scheme.
        //Returns 0 for NumArgs=0.

	int ar;
 
	int value=0;
	for(ar=m_NumArgs-1; ar>=0; ar--) {
	    value = size*value+m_args[ar];
	}
	return(value);
    }

    public void setNumValue(int value, int size) {
	//sets the arguments as the "digits"
	//of "value" as a base "size" number.
	//It is ASSUMED that size > 0
	//Does nothing for NumArgs=0.

	int ar;
	
	for(ar=0; ar<m_NumArgs; ar++) {
	    m_args[ar] = value % size;
	    value /= size;
	}
    } 

    public boolean incr_arguments(int size ) {
	//Returns true if succesful, false if this is the biggest argument,
	//in this case everything is set to zero.

	int i;
 
	for(i=0; i<m_NumArgs; i++) {
	    (m_args[i])++;
	    if (m_args[i] >= size) {
		m_args[i] = 0;
		continue;  /* get next coordinate */
	    }
	    return(true);      /* m_args[i] has been increased */
	}
	return(false); /* all arguments done */
    }

    public String toString() {
	//Returns the empty string when m_NumArgs=0.

	StringBuffer buf = new StringBuffer("");
	int i;
	for(i=0; i<m_NumArgs; i++) {
	    buf.append(m_args[i]);
	    if(i<m_NumArgs-1) {
		buf.append(", ");
	    }
	}
	return(buf.toString());
    }

    public String toString(Operation f) {
	String s = new String(f.getNameString() +
			      "(" + toString() + ")");

	return s;
    }

    private int m_NumArgs;
    private int m_args[];
    
}
