package uacalc.alg;

import uacalc.alg.con.Partition;
import uacalc.util.ewkInputStream;
import uacalc.vectors.IntArray;
import uacalc.vectors.VectorList;
import uacalc.util.ewkGetNumberDialog;
import uacalc.alg.opedit.OpValueReader;

import uacalc.ex.BadAlgebraException;
import uacalc.ex.BadOperationException;
import uacalc.ex.OperationWriteException;
import uacalc.ex.TooManyArgumentsException;

import java.io.*;
import java.util.*;
import java.awt.*;
import javax.swing.*;

public class Operation {

    public Operation(int size, int name) {
	m_size = size;
	m_name = name;
	m_arity = -1;
	m_valid = false;
	isModified = false;
    }

/*
    public void copy(Operation f) throws BadOperationException {
	m_arity = f.m_arity;
	m_name = f.m_name;
	m_NumValues = f.m_NumValues;
	try {
	    m_values = new int[m_NumValues];
	}
	catch(OutOfMemoryError e) {
	    throw new BadOperationException(
		" ! Not enough memory to store operation " 
		+ getNameString() + ".");
	} 
	for(int i=0; i< m_NumValues; i++) {
	    m_values[i] = f.m_values[i];
	}
	m_valid = f.m_valid;
	isModified = f.isModified;
    }
*/
    public int getSize() {
	return m_size;
    }

    public boolean tryValue(int [] args, int value) {

	boolean flag = validateValue(value);
	if(flag) {
	    setValue(args, value);
	}
	return flag;
    }

    public boolean tryValue(OpArguments Args, int value) {

	boolean flag = validateValue(value);
	if(flag) {
	    setValue(Args, value);
	}
	return flag;
    }

    public boolean tryValue(int arg0, int arg1, int value) {

	boolean flag = validateValue(value);
	if(flag) {
	    setValue(arg0, arg1, value);
	}
	return flag;
    }

    public void setValue(OpArguments Args, int value) {
	//Assumes that value is valid

	isModified = true;
	m_values[Args.getNumValue(m_size)]=value;
    }

    public void setValue(int [] args, int value) {

	isModified = true;
	m_values[args_to_lin(args)]=value;
    }

    public void setValue(int arg0, int arg1, int value) {
	//For binary operations this is simpler and faster.
	//Also DANGEROUS: watch out for changes in OpArguments.java
	isModified = true;
	m_values[arg1*m_size + arg0] = value;
    }

    public int getValue(OpArguments Args) {
	return(m_values[Args.getNumValue(m_size)]);
    }

    public int getValue(int [] args) {
//It is assumed that arguments[] have length m_arity
	return(m_values[args_to_lin(args)]);
    }
    
    public int getValue(int arg0, int arg1) {
	//For binary operations this is simpler and faster:
	//Also DANGEROUS: watch out for changes in OpArguments.java
	return(m_values[arg1*m_size + arg0]);
    }

    public void setArity(int arity) throws BadOperationException {
//Destructs the original operation even on error!

	m_valid = false;
	if(arity < 0 || arity >= ARITY_LIMIT) {
	    throw new BadOperationException(
		" ! Illegal arity " + arity +" (should be in [" +
		0 + "," +  (ARITY_LIMIT-1) + "]).");
	}
	m_NumValues = GetNumberOfValues(m_size, arity);
	if(m_NumValues == -1) {
	    throw new BadOperationException("(Algebra size)^(arity) too big.");
	}
	try {
	    m_values = new int[m_NumValues];
	}
	catch(OutOfMemoryError e) {
	    throw new BadOperationException(
		" ! Not enough memory to store operation " 
		+ getNameString() + ".");
	} 
//For safety we initialize to -1. We can live with this tiny overhead.
	int i;
	for( i=0; i<m_NumValues; i++) {
	    m_values[i]=-1;
	}
	m_arity = arity;
//Operation stays invalid until all values are filled!
    }

    public int getArity() {
	return m_arity;
    }

    public int getName() {
	return m_name;
    }

    public String getNameString() {
	return "f" + m_name;
    }

    public void setName(int name) {
	m_name = name;
    }

    public boolean isValid() {
	return m_valid;
    }

    public void setValid(boolean v) {
	//This should be done if all values are filled
	//Called by OpValueReader at the moment.
	m_valid = v;
    }

    public int getNumValues() {
	//It is assumed that the operation is valid.
	return m_NumValues;
    }


    public boolean Read( ewkInputStream fp ) throws BadOperationException {
//Reads the next function from fp. Returns true if succesful,
//false if there are no more functions.
//Dialogs are not shown here, that is the reponsibility
//of the caller.
//The file is NOT closed.

	int ar = fp.get_integer("\n\r");
	if( ar == -2) return(false);//EOF, no more ops in the file.
	if( ar == -1) {
	    throw new BadOperationException(
		" ! Error reading arity of operation (bad delimiter?).");
	}
	if(ar < 0 || ar >= ARITY_LIMIT) // arity <0 is impossible anyway
	{
	    throw new BadOperationException(
		" ! Illegal arity " + ar +" (should be in [" +
		0 + "," +  (ARITY_LIMIT-1) + "]).");
	}
	setArity(ar);

	int i;
	for ( i=0; i<m_NumValues; i++)
	{
	    int in = fp.get_integer("\n\r "); /* space can also delimit values */
	    if( in == -1 || in == -2)
	    {
		throw new BadOperationException(
		    " ! Error reading value of operation (bad delimiter?).");
	    }
	    if( in < 0 || in >= m_size)
	    {
		throw new BadOperationException(
		    " ! Illegal operation value " + ar +" read (should be in [" +
		    0 + "," +  (m_size-1) + "]).");
	    }
	    m_values[i] = in;
	}
	m_valid = true;
	isModified = false;
	return(true);
    } 

    public void Write(PrintStream fp) 
	throws OperationWriteException {
	//Writes operation to an already open file.

	fp.println(m_arity);
	int i;
	for(i=0; i<m_NumValues; i++) {
	    fp.println(m_values[i]);
	}
	if( fp.checkError() ) {
	    throw new OperationWriteException(
		" ! Error writing operation " + getNameString() + ".");
	}
	isModified = false;
    }
  
    public String toString() {

	String out = "";

	out += "Operation " + getNameString() +
	    " has arity " + m_arity + ".\n";
	return(out);
    }

    public boolean Input(Frame parent) {

	while(true) {
	    ewkGetNumberDialog dialog = new ewkGetNumberDialog(
		parent,
		"Enter the arity of the new operation.",
		-1, 0, Operation.ARITY_LIMIT);
	    int arity = dialog.getValidatedNumber();
	    if(arity == -1) {
		int ans = JOptionPane.showConfirmDialog(
		    parent, 
		    "Are you sure you want to abandon adding a new operation?",
		    "Confirm",
		    JOptionPane.YES_NO_OPTION,
		    JOptionPane.QUESTION_MESSAGE);
		if(ans != JOptionPane.YES_OPTION) continue;
		return(false);
	    }
	    try {
		setArity(arity);
	    } catch(BadOperationException e) {
		JOptionPane.showMessageDialog(
		    parent, 
		    e.toString(),
		    "Arity Error",
		    JOptionPane.ERROR_MESSAGE);
		continue;
	    }
	    break;
	}
	OpValueReader reader = 
	    new OpValueReader(parent, this);
	if(!isValid()) {
	    //User already confirmed cancel
	    return(false);
	}
	isModified = true;
	return(true);
    }

//
// Private stuff
//

    private boolean validateValue(int value) {
	if(value < 0 || value >= m_size) return false;
	return true;
    }

    private int GetNumberOfValues(int size, int arity) {
	double d;

	d = Math.pow(size, arity);
	if(d >= Integer.MAX_VALUE) return(-1);
	return((int)d);
    }

    private int args_to_lin(int[] args) {
// returns the "value" of args as a base "size" number. Uses the Horner scheme.
	{
	    int ar, offset;
	    
	    offset=0;
	    for(ar=m_arity-1; ar>=0; ar--) {
		offset = m_size*offset+args[ar];
	    }
	    return(offset);
	}
    } 

    public boolean isModified() {
	return isModified;
    }

    public Operation factor(Partition part, int name) {
	int blockNum = part.getNumberOfBlocks();
	Operation g = new Operation(blockNum, name);
	int[] matt = part.toMatt();
	int[] repr = new int[blockNum];
	int i;
	for(i=0; i<m_size; i++) {
	    //Maybe there is a faster way, doesn't matter.
	    //This finds the biggest representative of every block.
	    repr[matt[i]]=i;
	}
	try {
	    g.setArity(m_arity);
	} catch(BadOperationException e) {
	    System.out.println("Impossible BadOperationException");
	    System.exit(0);
	}

	OpArguments Argsf = null;
	try {
	    Argsf = new OpArguments(m_arity);
	} catch(TooManyArgumentsException e) {
	    System.out.println("Impossible TooManyArgumentsException");
	    System.exit(0);
	}

	int[] Args = new int[m_arity];
	while(true) {
	    for(i=0; i<m_arity; i++) {
		Args[i] = repr[Argsf.getValue(i)];
	    }
	    g.setValue(Argsf, matt[getValue(Args)]);
	    if( !Argsf.incr_arguments(blockNum) ) break;
	}
	g.setValid(true);
	return(g);
    }

    public static Operation directProduct(
	ArrayList components, VectorList universe, int name) 
	throws BadAlgebraException {

//I am not sure if this is needed, but to increase
//efficiency, I put the generators into a HashMap
//to make lookup easier.

	int size = universe.size();
	HashMap uniHash = new HashMap(size);
	Iterator it = universe.iterator();
	IntArray vec;
	int i=0;
	while( it.hasNext() ) {
	    vec = new IntArray((int[])it.next());
	    if( !uniHash.containsKey(vec) ) {
		uniHash.put(vec, new Integer(i) );
		i++;
	    } else {
		throw new BadAlgebraException(
		" ! Duplicate entry in universe.");
	    }
	}

	int len = components.size();
	Operation g = new Operation(size, name);
	int arity = ((Operation)components.get(0)).getArity();

	try {
	    g.setArity(arity);
	} catch(BadOperationException e) {
	    System.out.println("Impossible BadOperationException");
	    System.exit(0);
	}

	OpArguments Args = null;
	try {
	    Args = new OpArguments(arity);
	} catch(TooManyArgumentsException e) {
	    System.out.println("Impossible TooManyArgumentsException");
	    System.exit(0);
	}

	int j;
	IntArray values = new IntArray(len);
	int[] Argsi = new int[arity];
	while(true) {//Cycle through arguments
	    for(i=0; i<len; i++) {//through components
		for(j=0; j<arity; j++) {//set i-th arglist
		    Argsi[j] = ((int[])universe.get(Args.getValue(j)))[i];
		}
		values.set(i, ((Operation)components.get(i)).getValue(Argsi));
	    }
	    Object o = uniHash.get(values);
	    if( o == null ) {
		throw new BadAlgebraException(
		    " ! This list of vectors is not a subuniverse.");
	    }
	    g.setValue(Args, ((Integer)o).intValue());
	    if( !Args.incr_arguments(size) ) break;
	}
	g.setValid(true);
	return(g);
    }



//Max value of an arity:
    public static final int ARITY_LIMIT = 10;

    protected int m_size;
    protected int m_arity;
    protected int m_NumValues;
    protected int[] m_values;
    private int m_name;//Presently this is a non-negative integer as in g23
    private boolean m_valid;//This is false unless the values
                            //are filled in in an allowed way.
    private boolean isModified;//Values
    
//Invalid if size^arity is too big,
//in that case all attributes are undefined.

}

