package uacalc.alg.algedit;

import uacalc.alg.Algebra;
import uacalc.alg.Operation;
import uacalc.util.WholeNumberField;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.beans.*;
import java.util.*;

//This is a dialog with a WholeNumberField and a ComboBox.
//It lets the user select an operation from the ComboBox,
//or he can type the number of the operation.

class OpChooser {

    public OpChooser(JFrame parent, Algebra A, 
		     String message, int value ) {

//value is the default value.
//set it to -1 to display the empty string initially.
//it is assumed that "value" is within range.
//WARNING: DO NOT USE -1 for value, it looks bad,
//because the combo box shall have an empty text,
//and this does not orient the user.

	m_max = A.getNumOp();
	if(m_max == 1) {
	    m_value = 0;
	    return;
	}
	m_value= value;
	m_min = 0;

	dialog = new JDialog(parent, true);
	//This doesn't work for some reason, the dialog is not
	//centered over this frame. A partial solution is the next command:
	dialog.setLocationRelativeTo(parent);
        dialog.setTitle("Select Operation");

	String message1 = 
	    "\nEITHER enter the number of the operation you want to select, ";
        final WholeNumberField NumberField = 
	    new WholeNumberField(m_value, 0);
	String message2 = 
	    "\nOR select an operation using the mouse by clicking below.";
	Vector OpList = new Vector();
        int i;
        Operation f;
        for( i=0; i< m_max; i++) {
            f = A.getOp(i);
            OpList.add(" "+ f.getNameString() + " has arity " +
                          f.getArity() + " ");
        }

	final JComboBox ComboBox = new JComboBox(OpList);
	ComboBox.setSelectedIndex(value);

        Object[] array = {message, message1, NumberField, message2, ComboBox};

        final String btnString1 = "Done";
        final String btnString2 = "Cancel";
        Object[] options = {btnString1, btnString2};

        optionPane = new JOptionPane(array, 
				     JOptionPane.QUESTION_MESSAGE,
				     JOptionPane.OK_CANCEL_OPTION,
				     null,
				     options,
				     options[0]);
        dialog.setContentPane(optionPane);
        dialog.setDefaultCloseOperation(dialog.DO_NOTHING_ON_CLOSE);
	optionPane.setValue(
	    JOptionPane.UNINITIALIZED_VALUE);

        dialog.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent we) {
                /*
                 * Instead of directly closing the window,
                 * we're going to change the JOptionPane's
                 * value property.
                 */
                    optionPane.setValue(new Integer(
                                        JOptionPane.CLOSED_OPTION));
		}
		public void windowActivated(WindowEvent e) {
		    NumberField.requestFocus();
		}
        });

        NumberField.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    optionPane.setValue(btnString1);
		}
        });

        ComboBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    JComboBox cb = (JComboBox)e.getSource();
		    m_value = cb.getSelectedIndex();
		    NumberField.setValue(m_value);
		    //This is the simplest solution, because then
		    //we only have to listen to the value of the NumberField.
		    optionPane.setValue(btnString1);
		}
        });

        optionPane.addPropertyChangeListener(new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent e) {
                String prop = e.getPropertyName();

                if (dialog.isVisible() 
                 && (e.getSource() == optionPane)
                 && (prop.equals(JOptionPane.VALUE_PROPERTY) ||
                     prop.equals(JOptionPane.INPUT_VALUE_PROPERTY))) {
                    Object v = optionPane.getValue();

                    if (v == JOptionPane.UNINITIALIZED_VALUE) {
                        //ignore reset
                        return;
                    }

                    // Reset the JOptionPane's value.
                    // If you don't do this, then if the user
                    // presses the same button next time, no
                    // property change event will be fired.
                    optionPane.setValue(
                            JOptionPane.UNINITIALIZED_VALUE);

                    if (v.equals(btnString1)) {
                            m_value = NumberField.getValue();
                        if (m_value >= m_min && m_value <= m_max) {
                            // we're done; dismiss the dialog
                            dialog.setVisible(false);
                        } else { 
                            // number was invalid
                            NumberField.selectAll();
			    NumberField.requestFocus();
                            JOptionPane.showMessageDialog(
                                            dialog,
					    "Please enter a number between " +
					    m_min + " and " + m_max + ",\n" +
					    "or click on an operation.",
					    "Number out of bounds",
                                            JOptionPane.ERROR_MESSAGE);
                        }
                    } else { 
                        // user closed dialog or clicked cancel
			m_value = -1;
			dialog.setVisible(false);
                    }
                }
            }
        });

	dialog.pack();
	NumberField.selectAll();
        dialog.setVisible(true);
    }

    public int getSelectedOp() {
        return m_value;
    }

    private int m_value;
    private int m_max;
    private int m_min;
    private JOptionPane optionPane;
    private JDialog dialog;

}
