package uacalc.alg.con.ui;

import uacalc.vectors.BinaryRelation;
import uacalc.alg.Algebra;
import uacalc.alg.con.CongruenceLattice;
import uacalc.alg.con.Partition;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.beans.*;

public class BinRelSelector extends JDialog {

    public BinRelSelector(
	JFrame parent, String message,
	CongruenceLattice congs) {

        super(parent, true);
	//This doesn't work for some reason, the dialog is not
	//centered over this frame. A partial solution is the next command:
	setLocationRelativeTo(parent);

        setTitle("Select a binary relation");
	m_parent = parent;
	m_A = congs.getAlgebra();
	m_congs = congs;

	m_size = congs.getSize();
	cancelled = false;
	m_binrel = new BinaryRelation();

	Object[] array = {message};

        final String btnString1 = "Cancel";
        final String btnString2 = "Import congruence";
        final String btnString3 = "Generate";

        Object[] options = {btnString1, btnString2, btnString3};

        optionPane = new JOptionPane(array, 
				     JOptionPane.QUESTION_MESSAGE,
				     JOptionPane.OK_CANCEL_OPTION,
				     null,
				     options,
				     options[1]);
        setContentPane(optionPane);
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
	optionPane.setValue(
	    JOptionPane.UNINITIALIZED_VALUE);

        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent we) {
                /*
                 * Instead of directly closing the window,
                 * we're going to change the JOptionPane's
                 * value property.
                 */
                    optionPane.setValue(new Integer(
                                        JOptionPane.CLOSED_OPTION));
		}
        });

        optionPane.addPropertyChangeListener(new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent e) {
                String prop = e.getPropertyName();

                if (isVisible() 
                 && (e.getSource() == optionPane)
                 && (prop.equals(JOptionPane.VALUE_PROPERTY) ||
                     prop.equals(JOptionPane.INPUT_VALUE_PROPERTY))) {
                    Object v = optionPane.getValue();

                    if (v == JOptionPane.UNINITIALIZED_VALUE) {
                        //ignore reset
                        return;
                    }

                    // Reset the JOptionPane's value.
                    // If you don't do this, then if the user
                    // presses the same button next time, no
                    // property change event will be fired.
                    optionPane.setValue(
                            JOptionPane.UNINITIALIZED_VALUE);

                    if (v.equals(btnString2)) {
			OnSelectCongruence();
		    } else if (v.equals(btnString3)) {
			OnGenerate();
                    } else { 
                        // user closed dialog or clicked cancel
			cancelled = true;
			setVisible(false);
                    }
                }
            }
        });
	pack();
        setVisible(true);
    }

    public boolean isCancelled() {
	return cancelled;
    }

    public BinaryRelation getSelectedBinRel() {
        return m_binrel;
    }

    public void OnSelectCongruence() {
	CongruenceSelector dialog = new CongruenceSelector(
            m_parent,
	    "Please, select a congruence.",
            -1, m_congs, true, true, true, false);
	if ( dialog.isCancelled() ) {
	    cancelled = true;
	    setVisible(false);
	}
	if( m_congs.getSize() == -1 ) {
	    m_binrel = new BinaryRelation(
		dialog.getSelectedPartition());
	} else {
	    m_binrel = new BinaryRelation(
		m_congs, dialog.getValidatedNumber());
	}
	setVisible(false);
    }

    private void OnGenerate() {
	int algSize = m_A.getSize();
	int[] limits = {algSize, algSize};
        if( !m_binrel.Input(m_parent, limits,
			    "Please enter generator") ) {
	    cancelled = true;
	}
	setVisible(false);
    }


    private JOptionPane optionPane;

    private JFrame m_parent;
    private Algebra m_A;
    private int m_size;
    private CongruenceLattice m_congs;

    private BinaryRelation m_binrel;
    private boolean cancelled;

}
