package uacalc.alg.con.ui;

import uacalc.alg.Algebra;
import uacalc.alg.con.CongruenceLattice;
import uacalc.alg.con.Partition;
import uacalc.vectors.BinaryRelation;

import uacalc.util.Project;
import uacalc.util.SwingWorker;
import uacalc.Progress;
import uacalc.Compute;

import uacalc.ex.AlgebraWriteException;

import java.awt.*;
import java.io.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;

public class Centrality {

    public Centrality(CongruenceDisplayerFrame parent,
		      JMenuBar menuBar,
		      CongruenceLattice congs) {

	m_parent = parent;
	m_congs = congs;
	m_A = congs.getAlgebra();

//Set up menus
	JMenu CentralityMenu = new JMenu("Centrality");
	CentralityMenu.setMnemonic(KeyEvent.VK_E);
	menuBar.add(CentralityMenu);

//Normal
        JMenuItem NormalMenuItem = 
	    new JMenuItem("Normal", KeyEvent.VK_N);
        CentralityMenu.add(NormalMenuItem);
        NormalMenuItem.setAccelerator(
            KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
 
        NormalMenuItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    OnCentrality(NORMAL);
                }
            });

//Strong
        JMenuItem StrongMenuItem = 
	    new JMenuItem("Strong", KeyEvent.VK_S);
        CentralityMenu.add(StrongMenuItem);
        StrongMenuItem.setAccelerator(
            KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
 
        StrongMenuItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    OnCentrality(STRONG);
                }
            });

//Rectangular
        JMenuItem RectangularMenuItem = 
	    new JMenuItem("Rectangular", KeyEvent.VK_R);
        CentralityMenu.add(RectangularMenuItem);
        RectangularMenuItem.setAccelerator(
            KeyStroke.getKeyStroke(KeyEvent.VK_R, ActionEvent.CTRL_MASK));
 
        RectangularMenuItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    OnCentrality(RECTANGULAR);
                }
            });

//Weak
        JMenuItem WeakMenuItem = 
	    new JMenuItem("Weak", KeyEvent.VK_W);
        CentralityMenu.add(WeakMenuItem);
        WeakMenuItem.setAccelerator(
            KeyStroke.getKeyStroke(KeyEvent.VK_W, ActionEvent.CTRL_MASK));
 
        WeakMenuItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    OnCentrality(WEAK);
                }
            });
    }


    private void OnCentrality(int type) {
	if( !Progress.ProgramStart() ) return;
	if( !ComputeCentrality(type) ) {
	    Progress.ProgramFinished();
	}
    }

    /**
     * Returns false if canceled, and on error,
     * true if the computation started.
     */
    private boolean ComputeCentrality(int type) {

	BinRelSelector dialog = new BinRelSelector(
	    m_parent,
	    "Please select \"left\" in " +
	    centralityName(type) + "(left, right; cong)",
	    m_congs);
	if ( dialog.isCancelled() ) return false;
	BinaryRelation left = dialog.getSelectedBinRel();

	dialog = new BinRelSelector(
	    m_parent,
	    "Please select \"right\" in " +
	    centralityName(type) + "(left, right; cong)",
	    m_congs);
	if ( dialog.isCancelled() ) return false;
	BinaryRelation right = dialog.getSelectedBinRel();

	String fn = m_A.getName();
	final Project fn_Project = new Project(fn);

	//Next we select "cong". There are the following possibilities.
	//If the congruence lattice is not computed, then only
	//cong = zero is allowed.
	//Otherwise the user can pick any congruence,
	//or can pick "all", in which case all congruences
	//are checked, and we stop at the first failure.

	String linec = null;
	int congNum = -1;
	if( m_congs.getSize() == -1 ) {
	    linec = "\\congruence{\\zero_congruence{}}";
	} else {
	    CongruenceSelector dialogc = new CongruenceSelector(
		m_parent,
		"Please select \"cong\" in " +
		centralityName(type) + "(left, right; cong)",
		-1, m_congs, true, false, true, true);
	    if ( dialogc.isCancelled() ) return false;
	    congNum = dialogc.getValidatedNumber();
	    if( congNum == CongruenceSelector.ALL ) {
		linec = "\\conlist_file{" +
		    fn_Project.getName(Project.ConListType) + "}";
	    } else {
		linec = "\\congruence{\\conlist_file{" +
		    fn_Project.getName(Project.ConListType) + 
		    "}\\number{" + congNum + "}}";
	    }
	}

	String line1 = left.prepareBinRelLine(
	    fn_Project, Project.BinRelType1);
	String line2 = right.prepareBinRelLine(
	    fn_Project, Project.BinRelType2);

	try {
	    FileOutputStream fout =
		new FileOutputStream(
		    fn_Project.getName(Project.ParType));
	    PrintStream fp =
		new PrintStream(fout);
	    fp.println("\\begin_command");
	    switch(type) {
		case NORMAL:
		    fp.println("\\centrality");
		    break;
		case STRONG:
		    fp.println("\\strong_centrality");
		    break;
		case RECTANGULAR:
		    fp.println("\\rectangular_centrality");
		    break;
		case WEAK:
		    fp.println("\\weak_centrality");
		    break;
	    }
	    fp.println("\\algebra_file{" +
		       fn_Project.getName(Project.AlgebraType) + "}");
	    fp.println(line1);
	    fp.println(line2);
	    fp.println(linec);
	    fp.println("\\end_command");
	    fp.close();
	    fout.close();
	    if( fp.checkError() ) {
		throw new AlgebraWriteException(//!!!!!!!!!!!!!!!!
		    "Error writing parameter file\n" +
		    fn_Project.getName(Project.ParType) + "\n");
	    }
	}
	catch(Exception e) {
	    Progress.ProgressLogAppend(e.toString());	    
	}
	worker = new SwingWorker() {
		public Object construct() {
		    Compute.run_uab(
			fn_Project.getName(Project.NoType), Compute.defaultLogLevel);
		    return "";
		}
		public void finished() {
		    Progress.ProgramFinished();
		}
	    };

//Write to result places
	Progress.ResultLogAppend(
	    "\n [ Algebra: " +  m_A.getName());

	String res = 
	    "\n [ Computing centrality " +
	    centralityName(type) + "(left, right; cong)\n" +
	    "left = " + left.toString() + "\n" +
	    "right = " + right.toString() + "\n" +
	    "cong = ";
	
	if( m_congs.getSize() == -1 ) {
	    res += m_congs.getZeroCongruencePartition().
		toString(Partition.HUMAN) + "\n";
	} else {
	    if( congNum == CongruenceSelector.ALL ) {
		res += "all congruences\n";
	    } else {
		res += m_congs.getPartition(congNum).
		    toString(Partition.HUMAN) + "\n";
	    }
	}
	fn_Project.appendToLog(res);
	worker.start();  //required for SwingWorker 3
	return true;
    }


    public String centralityName(int type) {
	switch(type) {
	    case NORMAL:
		return "C";
	    case STRONG:
		return "C*";
	    case RECTANGULAR:
		return "CR";
	    case WEAK:
		return "CW";
	}
	return "Cunknown";
    }

    private CongruenceLattice m_congs;
    private CongruenceDisplayerFrame m_parent;
    private Algebra m_A;

    private static final int NORMAL = 0;
    private static final int STRONG = 1;
    private static final int RECTANGULAR = 2;
    private static final int WEAK = 3;
    private SwingWorker worker;

}
