package uacalc.alg.con.ui;

import uacalc.alg.Algebra;
import uacalc.alg.con.CongruenceLattice;
import uacalc.alg.con.Partition;
import uacalc.alg.con.TypeFinder;
import uacalc.alg.con.MinimalSetList;

import uacalc.alg.algedit.AlgebraDisplayerPane;
import uacalc.util.SwingWorker;
import uacalc.util.Project;
import uacalc.util.ewkFileChooser;
import uacalc.config.Configure;
import uacalc.vectors.VectorInput;
import uacalc.Progress;
import uacalc.Compute;

import uacalc.ex.AlgebraWriteException;

import java.awt.*;
import java.io.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import java.text.*;

public class CongruenceDisplayerFrame extends JFrame {
//We do not assume that C1 is the one congruence.
//We do not even assume that C0 is zero.

    public CongruenceDisplayerFrame(JFrame parent,
				    CongruenceLattice congs) {
	
	m_parent = parent;
	m_congs = congs;
	m_A = m_congs.getAlgebra();
	algSize = m_A.getSize();
	if( algSize == 1 ) return;
	setTitle("View and manipulate congruences");
	integerFormatter = NumberFormat.getNumberInstance(Locale.US);
        integerFormatter.setParseIntegerOnly(true);

//Set up menus
        menuBar = new JMenuBar();
        setJMenuBar(menuBar);
 
//File menu
        JMenu FileMenu = new JMenu("File");
        FileMenu.setMnemonic(KeyEvent.VK_F);
        menuBar.add(FileMenu);
 
//Close
        JMenuItem CloseMenuItem = new JMenuItem("Close", KeyEvent.VK_C);
        FileMenu.add(CloseMenuItem);
        CloseMenuItem.setAccelerator(
            KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK));
 
        CloseMenuItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    OnClose();
                }
            });

	setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    OnClose();
                }
            });

//Select menu
        JMenu SelectMenu = new JMenu("Select");
        SelectMenu.setMnemonic(KeyEvent.VK_S);
        menuBar.add(SelectMenu);
 
//Congruence
        JMenuItem CongruenceMenuItem = 
	    new JMenuItem("Select/generate congruence", KeyEvent.VK_U);
        SelectMenu.add(CongruenceMenuItem);
        CongruenceMenuItem.setAccelerator(
            KeyStroke.getKeyStroke(KeyEvent.VK_U, ActionEvent.CTRL_MASK));
 
        CongruenceMenuItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    OnCongruence();
                }
            });

//Principal
        PrincipalMenuItem = 
	    new JMenuItem("Select principal congruence", KeyEvent.VK_P);
        SelectMenu.add(PrincipalMenuItem);
        PrincipalMenuItem.setAccelerator(
            KeyStroke.getKeyStroke(KeyEvent.VK_P, ActionEvent.CTRL_MASK));
 
        PrincipalMenuItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    OnPrincipal();
                }
            });

//Compute factor
        JMenuItem FactorMenuItem = 
	    new JMenuItem("Compute factor algebra", KeyEvent.VK_O);
        SelectMenu.add(FactorMenuItem);
        FactorMenuItem.setAccelerator(
            KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
 
        FactorMenuItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    OnFactor();
                }
            });

//Lattice menu
        JMenu ComputeMenu = new JMenu("Lattice");
        ComputeMenu.setMnemonic(KeyEvent.VK_L);
        menuBar.add(ComputeMenu);
 
//Generate
        GenerateMenuItem = 
	    new JMenuItem("Compute and label", KeyEvent.VK_A);
        ComputeMenu.add(GenerateMenuItem);
        GenerateMenuItem.setAccelerator(
            KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK));
 
        GenerateMenuItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    OnGenerate();
                }
            });


//Join
        JMenuItem JoinMenuItem = 
	    new JMenuItem("Join", KeyEvent.VK_J);
        ComputeMenu.add(JoinMenuItem);
        JoinMenuItem.setAccelerator(
            KeyStroke.getKeyStroke(KeyEvent.VK_J, ActionEvent.CTRL_MASK));
 
        JoinMenuItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    OnJoin();
                }
            });

//Meet
        JMenuItem MeetMenuItem = 
	    new JMenuItem("Meet", KeyEvent.VK_M);
        ComputeMenu.add(MeetMenuItem);
        MeetMenuItem.setAccelerator(
            KeyStroke.getKeyStroke(KeyEvent.VK_M, ActionEvent.CTRL_MASK));
 
        MeetMenuItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    OnMeet();
                }
            });

//TCTSub submenu
	JMenu TCTMenu = new JMenu("Taming");
	TCTMenu.setMnemonic(KeyEvent.VK_T);
	menuBar.add(TCTMenu);

//TypeSet
        TypeSetMenuItem = 
	    new JMenuItem("Compute type set", KeyEvent.VK_T);
	TCTMenu.add(TypeSetMenuItem);
	TypeSetMenuItem.setAccelerator(
	    KeyStroke.getKeyStroke(KeyEvent.VK_T, ActionEvent.CTRL_MASK));

	TypeSetMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    OnTypeSet();
		}
	    });

//MinSets
        MinSetsMenuItem = 
	    new JMenuItem("Minimal sets", KeyEvent.VK_I);
	TCTMenu.add(MinSetsMenuItem);
	MinSetsMenuItem.setAccelerator(
	    KeyStroke.getKeyStroke(KeyEvent.VK_I, ActionEvent.CTRL_MASK));

	MinSetsMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    OnMinSets();
		}
	    });

//Centrality menu
	m_centrality = new Centrality(this, menuBar, m_congs);


//Graphics Menu
	JMenu GraphicsMenu = new JMenu("Graphics");
	GraphicsMenu.setMnemonic(KeyEvent.VK_G);
	menuBar.add(GraphicsMenu);

//DispLat
        DispLatMenuItem = new JMenuItem("Draw Lattice", KeyEvent.VK_D);
	GraphicsMenu.add(DispLatMenuItem);
	DispLatMenuItem.setAccelerator(
	    KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.CTRL_MASK));

	DispLatMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    OnDispLat();
		}
	    });



//Set up window components
 
        pane = getContentPane();
 
        pane.setLayout(new BoxLayout(pane, BoxLayout.X_AXIS));
 
        leftPane = new JPanel();
        leftPane.setLayout(new BoxLayout(leftPane, BoxLayout.Y_AXIS));
        leftPane.setAlignmentY(Component.TOP_ALIGNMENT);
        Configure.createBorder(leftPane);
 
        rightPane = new JPanel();
        rightPane.setLayout(new BoxLayout(rightPane, BoxLayout.Y_AXIS));
        rightPane.setAlignmentY(Component.TOP_ALIGNMENT);
        Configure.createBorder(rightPane);
 
//Show algebra
	algebraPane = new AlgebraDisplayerPane(leftPane, rightPane);
	algebraPane.setAlgebra(m_A);

//Put it together
	panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));

        panel.add(leftPane);
        panel.add(rightPane);
	scroll = new JScrollPane(panel);

	if( m_congs.getSize() == -1 ) {
	    PrincipalMenuItem.setEnabled(false);
	    MinSetsMenuItem.setEnabled(false);
	}

	if( m_congs.isTCTlabeled() ) {
	    TypeSetMenuItem.setEnabled(false);
	    GenerateMenuItem.setEnabled(false);
	} else {
	    DispLatMenuItem.setEnabled(false);
	}

	m_displayer = null;
	showCongruenceNumbers();
	setScrollSize();
	pack();
	setVisible(true);

    }
//////////////////
//endOfConstructor

    private void showCongruenceNumbers() {

	if( m_congs.getSize() != -1) {
	    //Set up Displayer
	    m_displayer = new CongruenceLatticeDisplayer(
		this, leftPane, rightPane, m_congs);
	    setScrollSize();
	    pack();
	}
    }


    private void setScrollSize() {
//The following is a general problem concerning scrollbars.
//Suppose that the width of an area changes, and therefore
//a horizontal scrollbar appears. Then the width of this
//scrollbar increases the height, and therefore a vertical
//scrollbar appears, too. This is not nice behaviour.
//The solution that I did is to increase the Preferred height by
//the width of the scroll bar. but this is not nice either,
//because then the original height is bigger than the natural
//height with the width of the scrollbar even when a scrollbar
//is not present, that is, there is an ugly margin at the bottom.

	int w;
	int h;
	w = (int)panel.getPreferredSize().getWidth();
	h = (int)panel.getPreferredSize().getHeight();
        w += (int)scroll.getVerticalScrollBar().getMinimumSize().getWidth();
        h += (int)scroll.getHorizontalScrollBar().getMinimumSize().getHeight();
	Insets ins = scroll.getBorder().getBorderInsets(scroll);
	w += ins.left + ins.right;
	h += ins.top + ins.bottom;
        Dimension MaxScrollDim = new Dimension();
	MaxScrollDim.setSize(w, h);
	scroll.setPreferredSize(MaxScrollDim);
	pane.add(scroll);

	leftPane.setMaximumSize(leftPane.getPreferredSize());
    }

/////////////////
//Menu callbacks

    private void OnClose() {
	if( m_displayer != null ) {
	    m_displayer.disposeDrawer();
	}
        dispose();
    }

    private void OnCongruence() {

	boolean oneC = true;
	boolean zeroC = true;
	String s;
	int defaultCong;
	if( m_congs.getSize() == -1 ) {
	    oneC = false;
	    zeroC = false;
	    s = "You can select a congruence by generating it." ;
	    defaultCong = -1;
	} else {
	    s ="Please, enter the congruence number.";
	    defaultCong = m_displayer.getSelectedCongruence();
	}
	CongruenceSelector dialog = new CongruenceSelector(
            this,
	    s,
            defaultCong, m_congs, oneC, zeroC, true, false);
	if ( dialog.isCancelled() ) return;
	if( m_congs.getSize() == -1 ) {
	    return;
	}
        int cong = dialog.getValidatedNumber();
	m_displayer.setSelectedCongruence(
	    CongruenceLatticeDisplayer.ALL_CONGRUENCES, cong);
    }

    private void OnPrincipal() {

	int[] sizes = {algSize, algSize};
	int[] gen = {-1, -1};
	VectorInput dialog = new VectorInput(
	    this, 
	    gen,
	    sizes,
	    "Select generator",
	    "Enter the pair to be collapsed:");
	if ( dialog.isCancelled() ) return;
	m_displayer.setSelectedPrincipal(gen);
    }

    private void OnFactor() {

	String s;
	int defaultCong;
	if( m_congs.getSize() == -1 ) {
	    s = "You can select a congruence to mod out by generating it." ;
	    defaultCong = -1;
	} else {
	    s ="Please, enter the congruence number to mod out.";
	    defaultCong = m_displayer.getSelectedCongruence();
	}
	CongruenceSelector dialog = new CongruenceSelector(
            this,
	    s,
            defaultCong, m_congs, false, false, true, false);
	if ( dialog.isCancelled() ) return;
	Partition part = dialog.getSelectedPartition();

	String gen =
	ewkFileChooser.getFile(this, 
			       Project.getAlgebraFilter(),
			       "Name of the factoralgebra",
			       ewkFileChooser.NEW);
	if(gen == null) return;


	String fn = m_A.getName();
	Project fn_Project = new Project(fn);
	fn_Project.lock(true);

	Project gen_Project = new Project(gen);

	String message = 
	    "is the factor algebra of the algebra\n" +
	    fn + "\nby the congruence ";
	if( m_congs.getSize() != -1 ) {
	    message += "C" + dialog.getValidatedNumber() + " = ";
	}
	message += part.toString(Partition.HUMAN) + "\n";

	if( !gen_Project.create(
	    this, "This algebra " + message) ) {
	    fn_Project.lock(false);
	    return;
	}
	fn_Project.lock(false);

	//Name may have changed
	gen = gen_Project.getName(Project.AlgebraType);

	Algebra F = m_A.factor(part, gen);
	try {
	    FileOutputStream fout =
		new FileOutputStream(gen);
	    PrintStream fp =
		new PrintStream(fout);
	    F.Write(fp);
	    fp.close();
	    fout.close();
	}
	catch(Exception e) {
	    Progress.ProgressLogAppend("Save unsuccesful:\n" +
			       e.toString() + "\n");
	    return;
	}

	fn_Project.appendToLog(gen + "\n" + message + "\n");

    }


    private void OnGenerate() {

	if( !Progress.ProgramStart() ) return;
	worker = new SwingWorker() {
		public Object construct() {
		    c = pane.getCursor();
		    pane.setCursor(new Cursor(Cursor.WAIT_CURSOR));
		    m_congs.Complete();
		    return "";
		}
		public void finished() {
		    Progress.ProgramFinished();
		    pane.setCursor(c);
		    if( !m_congs.isTCTlabeled() ) return; //Error message??
		    
		    GenerateMenuItem.setEnabled(false);
		    PrincipalMenuItem.setEnabled(true);
		    TypeSetMenuItem.setEnabled(false);
		    MinSetsMenuItem.setEnabled(true);
		    DispLatMenuItem.setEnabled(true);
		    if( m_displayer == null ) {
			showCongruenceNumbers();
		    } else {
			m_displayer.displayTCTlabel();
		    }
		}
		private Cursor c;
	    };

	worker.start();  //required for SwingWorker 3
    }

    private void OnJoin() {
	String s;
	int cong1 = -1, cong2 = -1, congj;
	Partition part1 = m_congs.getZeroCongruencePartition();
	Partition part2 = m_congs.getZeroCongruencePartition();
	Partition partj;
	if( m_congs.getSize() == -1 ) {
	    s = "Please, select the first joinand by generating it.";
	} else {
	    s = "Please, select the first joinand.";
	}
	CongruenceSelector dialog = new CongruenceSelector(
            this,
	    s,
            -1, m_congs, false, false, true, false);
	if ( dialog.isCancelled() ) return;
	if( m_congs.getSize() == -1 ) {
	    part1 = dialog.getSelectedPartition();
	} else {
	    cong1 = dialog.getValidatedNumber();
	}

	if( m_congs.getSize() == -1 ) {
	    s = "Please, select the second joinand by generating it.";
	} else {
	    s = "Please, select the second joinand.";
	}
	dialog = new CongruenceSelector(
            this,
	    s,
            -1, m_congs, false, false, true, false);
	if ( dialog.isCancelled() ) return;
	Project m_Project = new Project(m_A.getName());
	if( m_congs.getSize() == -1 ) {
	    part2 = dialog.getSelectedPartition();
	    partj = part1.join(part2);
	    m_Project.appendToLog(
		" > The join is " + 
		partj.toString(Partition.HUMAN) + "\n");
	} else {
	    cong2 = dialog.getValidatedNumber();
	    congj = m_congs.getJoin(cong1, cong2);
	    m_Project.appendToLog(
		" > The join of C" + cong1 + "and C" + cong2 + 
		" is C" + congj + "\n");
	    m_displayer.setSelectedCongruence(
		CongruenceLatticeDisplayer.ALL_CONGRUENCES, congj);
	}
    }

    private void OnMeet() {
	String s;
	int cong1 = -1, cong2 = -1, congj;
	Partition part1 = m_congs.getZeroCongruencePartition();
	Partition part2 = m_congs.getZeroCongruencePartition();
	Partition partj;
	if( m_congs.getSize() == -1 ) {
	    s = "Please, select the first meetand by generating it.";
	} else {
	    s = "Please, select the first meetand.";
	}
	CongruenceSelector dialog = new CongruenceSelector(
            this,
	    s,
            -1, m_congs, false, false, true, false);
	if ( dialog.isCancelled() ) return;
	if( m_congs.getSize() == -1 ) {
	    part1 = dialog.getSelectedPartition();
	} else {
	    cong1 = dialog.getValidatedNumber();
	}

	if( m_congs.getSize() == -1 ) {
	    s = "Please, select the second meetand by generating it.";
	} else {
	    s = "Please, select the second meetand.";
	}
	dialog = new CongruenceSelector(
            this,
	    s,
            -1, m_congs, false, false, true, false);
	if ( dialog.isCancelled() ) return;
	Project m_Project = new Project(m_A.getName());
	if( m_congs.getSize() == -1 ) {
	    part2 = dialog.getSelectedPartition();
	    partj = part1.meet(part2);
	    m_Project.appendToLog(
		" > The meet is "+ 
		partj.toString(Partition.HUMAN) + "\n");
	} else {
	    cong2 = dialog.getValidatedNumber();
	    congj = m_congs.getMeet(cong1, cong2);
	    m_Project.appendToLog(
		" > The meet of C" + cong1 + "and C" + cong2 + 
		" is C" + congj + "\n");
	    m_displayer.setSelectedCongruence(
		CongruenceLatticeDisplayer.ALL_CONGRUENCES, congj);
	}
    }

    private void OnTypeSet() {

	if( !Progress.ProgramStart() ) return;
	Progress.ResultLogAppend(
	    "\n [ Computing the TCT type set of\n" +
	    m_A.getName());

	worker = new SwingWorker() {
		public Object construct() {
		    c = pane.getCursor();
		    pane.setCursor(new Cursor(Cursor.WAIT_CURSOR));
		    TypeFinder finder = new TypeFinder(m_congs);
		    typeSet = finder.findTypeSet();
		    return "";
		}
		public void finished() {
		    Progress.ProgramFinished();
		    pane.setCursor(c);
		    Project m_Project = new Project(m_A.getName());
		    m_Project.appendToLog(
			"\n > The TCT type set is " +
			typeSet.toString() + ".");
		    
		    TypeSetMenuItem.setEnabled(false);
		}
		private HashSet typeSet;
		private Cursor c;
	    };

	worker.start();  //required for SwingWorker 3
    }


    private void OnMinSets() {
	//At the moment this works only if the congruence lattice
	//is computed, and for simplicity, we only calculate
	//the minsets for the "current" cover.
	//This is a problem only if there are so many congruences
	//that it is uncomfortable to select one from a ComboBox.
	//Later, we also need the option to set a cover by
	//clicking on the lattice.

	if( !Progress.ProgramStart() ) return;

	final int congl;
	final int congu;
	String s;
	if( m_displayer.getUpperCoversAreShown() ) {
	    congl = m_displayer.getSelectedCongruence();
	    congu = m_displayer.getSelectedCover();
	} else {
	    congu = m_displayer.getSelectedCongruence();
	    congl = m_displayer.getSelectedCover();
	}

	final Project fn_Project = new Project(m_A.getName());
	try {
	    FileOutputStream fout =
		new FileOutputStream(
		    fn_Project.getName(Project.ParType));
	    PrintStream fp =
		new PrintStream(fout);
	    fp.println("\\begin_command");
	    fp.println("\\compute_min_sets");
	    fp.println("\\algebra_file{" +
		       fn_Project.getName(Project.AlgebraType) + "}");
	    fp.println("\\congruence{\\conlist_file{" +
		       fn_Project.getName(Project.ConListType) + 
		       "}\\number{" + congl + "}}");
	    fp.println("\\congruence{\\conlist_file{" +
		       fn_Project.getName(Project.ConListType) + 
		       "}\\number{" + congu + "}}");
	    fp.println("\\output_file{" + 
		       fn_Project.getName(Project.MinSetType) + "}");
	    fp.println("\\end_command");
	    fp.close();
	    fout.close();
	    if( fp.checkError() ) {
		throw new AlgebraWriteException(//!!!!!!!!!!!!!!!!
		    "Error writing parameter file\n" +
		    fn_Project.getName(Project.ParType) + "\n");
	    }
	}
	catch(Exception e) {
	    Progress.ProgressLogAppend(e.toString());	    
	}

	worker = new SwingWorker() {
		public Object construct() {
		    Compute.run_uab(
			fn_Project.getName(Project.NoType), Compute.defaultLogLevel);
		    return "";
		}
		public void finished() {
		    Progress.ProgramFinished();
		    if( !fn_Project.existsName(Project.MinSetType) ) {
			return;//Error message??
		    }
		    MinimalSetList minsets = new MinimalSetList();
		    if( !minsets.ReadVectorList(
			fn_Project.getName(Project.MinSetType),
			m_A.getSize(), Project.MinSetType )) {
			return;
		    }
		    fn_Project.appendToLog(
			"\n > The minimal sets are (the traces are in []):\n" +
			minsets.toString(
			    m_congs.getPartition(congl),
			    m_congs.getPartition(congu)	 ));
		}

	    };

//Write to result places
	Progress.ResultLogAppend(
	    "\n [ Algebra: " +  m_A.getName());

	String res = 
	    "\n [ Computing minimal sets for C" + congl +
	    " -< C" + congu + ".\nC" + congl + " = " +
	    m_congs.getPartition(congl).toString(Partition.HUMAN) +
	    "\nC" + congu + " = " +
	    m_congs.getPartition(congu).toString(Partition.HUMAN);
	fn_Project.appendToLog(res);

	worker.start();  //required for SwingWorker 3
    }

    private void OnDispLat() {

	String fn = m_A.getName();
	Project fn_Project = new Project(fn);
	m_congs.exportHasseDiagram();
        m_displayer.setLatticeDrawer(fn_Project);
	DispLatMenuItem.setEnabled(false);
    }

    public void OnDrawerClosed() {
	if( m_congs.isTCTlabeled() ) {//This is unnecessary, but maybe cleaner
	    DispLatMenuItem.setEnabled(true);
	}
    }


//Components
    private Container pane;
    private JPanel leftPane;
    private JPanel rightPane;
    private JPanel panel;
    private JScrollPane scroll;
    private AlgebraDisplayerPane algebraPane;
    private CongruenceLatticeDisplayer m_displayer;
    private JFrame m_parent;
    private Algebra m_A;
    private int algSize;
    private CongruenceLattice m_congs;

    private JMenuBar menuBar;
    private JMenuItem GenerateMenuItem;
    private JMenuItem PrincipalMenuItem;
    private JMenuItem TypeSetMenuItem;
    private JMenuItem MinSetsMenuItem;
    private JMenuItem DispLatMenuItem;
    private Centrality m_centrality;
    private SwingWorker worker;
    private NumberFormat integerFormatter;
}

