package uacalc.alg.con.ui;

import uacalc.alg.Algebra;
import uacalc.alg.con.CongruenceLattice;
import uacalc.alg.con.Partition;
import uacalc.util.Project;

import uacalc.config.Configure;


import java.awt.*;
import java.io.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

//TODO
//Treat the one element algebra differently
//Make a new class DrawerHighlighter, containing
//all the highlighting stuff.

public class CongruenceLatticeDisplayer {
//We do not assume that C1 is the one congruence.
//We do not even assume that C0 is zero.

    public CongruenceLatticeDisplayer(
	CongruenceDisplayerFrame parent,
	JPanel leftPane,
	JPanel rightPane,
	CongruenceLattice congs ) {
	
	m_parent = parent;
	m_leftPane = leftPane;
	m_rightPane = rightPane;
	m_congs = congs;
	m_A = congs.getAlgebra();
	algSize = m_A.getSize();
	if( algSize == 1 ) return;
	showCongruences();
    }

//////////////////
//endOfConstructor


    private void showCongruences() {

//make lists

	ConList = new Vector();
	Vector MIList = new Vector();
	Vector JIList = new Vector();
	mi = m_congs.getMINumbers();
	ji = m_congs.getJINumbers();
	zeroCongNum = m_congs.getZeroCongruence();
	oneCongNum = m_congs.getOneCongruence();

        intervalBottom = zeroCongNum;
        intervalTop = oneCongNum;
	drawingInterval = false;

//Interval shown
        IntervalTextLabel = new JLabel(
            "Interval drawn:");
        Configure.setupLabel(IntervalTextLabel);
        m_leftPane.add(IntervalTextLabel);
 
        IntervalValueLabel = new JLabel();
	setIntervalValueLabelText();
        Configure.setupLabel(IntervalValueLabel);
        m_rightPane.add(IntervalValueLabel);

        // rsf This is similar to ConList but has the drawer labels.
        //     Using this avoids recreating the label.
        drawerLabels = new String[m_congs.getSize()];

        Partition part;
	Iterator it = m_congs.getCongruenceIterator();
	int i = 0;
	while( it.hasNext() ) {
            drawerLabels[i] = CongruenceLattice.intToDrawerLabel(i);
	    part = (Partition)it.next();
	    int blocks = part.getNumberOfBlocks();
	    String s = " C" + i + " has " + blocks  + " blocks ";
            ConList.add(s);
	    if( m_congs.isMI(i) ) {
		MIList.add(s);
	    }
	    if( m_congs.isJI(i) ) {
		JIList.add(s);
	    }
	    i++;
        }

	Vector PrinList = new Vector();
	prin = new int[algSize*(algSize-1)/2];
	int a, b, num;
	i = 0;
	for(a=0; a<algSize-1; a++) {
	    for(b=a+1; b<algSize; b++) {
		num = m_congs.Cg(a,b);
		part = m_congs.getPartition(num);
		int blocks = part.getNumberOfBlocks();
		String s = " Cg(" + a + ", " + b +
		    ") = C" + num + " has " + blocks  + " blocks ";
		PrinList.add(s);
		prin[i] = num;
		i++;
	    }
	}

        Set distinctPrins = m_congs.getPrincipalNumbersAsSet();
        prinsAsSet = new int[distinctPrins.size()];
        i = 0;
        it = distinctPrins.iterator();
        while (it.hasNext()) {
          prinsAsSet[i] = ((Integer)it.next()).intValue();
          i++;
        }

//Selected congruence

	Vector ConMode = new Vector();
	ConMode.add(m_congs.getSize() + " congruences" +
		    " (see on the right)");
	ConMode.add(mi.length + " meet-irreducible congruence(s)" +
		    " (see on the right)");
	ConMode.add(ji.length + " join-irreducible congruence(s)" +
		    " (see on the right)");
	ConMode.add(m_congs.getNumberOfNonzeroPrincipals() +
		    " nonzero principal congruence(s)" +
		    " (see on the right)");

	ConModeBox = new ewkComboBox() {
		public void userSelected() {
		    SetMode( getSelectedIndex(), -1,
			     m_UpperCoversAreShown, -1 );
		}
	    };

	ConModeBox.setModelNA(new DefaultComboBoxModel(ConMode));
	ConModeBox.setSelectedIndexNA(ALL_CONGRUENCES);
	ConModeBox.setAlignmentX(Component.LEFT_ALIGNMENT);
        Configure.createBorder(ConModeBox);
        m_leftPane.add(ConModeBox);


//Congruence selector

	ConListModel = new DefaultComboBoxModel(ConList);
	MIListModel = new DefaultComboBoxModel(MIList);
	JIListModel = new DefaultComboBoxModel(JIList);
	PrinListModel = new DefaultComboBoxModel(PrinList);

	ConListBox = new ewkComboBox() {
		public void userSelected() {
		    SetCurrentCongruence( getSelectedIndex(),
					  m_UpperCoversAreShown, -1 );
		}
	    };

	ConListBox.setAlignmentX(Component.LEFT_ALIGNMENT);
        Configure.createBorder(ConListBox);
        m_rightPane.add(ConListBox);

//Partition
        PartitionTextLabel = new JLabel(
            "The corresponding partiton:");
        Configure.setupLabel(PartitionTextLabel);
        m_leftPane.add(PartitionTextLabel);
 
        PartitionValueLabel = new JLabel();
	Configure.setupLabel(PartitionValueLabel);
        m_rightPane.add(PartitionValueLabel);

//CoverMode ComboBox

	UpDownModeBox = new ewkComboBox() {
		public void userSelected() {
		    SetUpDownMode( getSelectedIndex(), -1 );
		}
	    };

	UpDownModeBox.setAlignmentX(Component.LEFT_ALIGNMENT);
        Configure.createBorder(UpDownModeBox);
        m_leftPane.add(UpDownModeBox);

//Coverlist ComboBox

	CoverListBox = new ewkComboBox() {
		public void userSelected() {
		    SetCurrentCongruence2( getSelectedIndex() );
		}
	    };

	CoverListBox.setAlignmentX(Component.LEFT_ALIGNMENT);
        Configure.createBorder(CoverListBox);
        m_rightPane.add(CoverListBox);


//Partition2
        Partition2TextLabel = new JLabel(
            "The corresponding partiton:");
        Configure.setupLabel(Partition2TextLabel);
        m_leftPane.add(Partition2TextLabel);
 
        Partition2ValueLabel = new JLabel();
	Configure.setupLabel(Partition2ValueLabel);
        m_rightPane.add(Partition2ValueLabel);


//TCTLabel
        TCTLabelTextLabel = new JLabel(
	    "TCT type label:");
        Configure.setupLabel(TCTLabelTextLabel);
        m_leftPane.add(TCTLabelTextLabel);
 
        TCTLabelValueLabel = new JLabel();
	Configure.setupLabel(TCTLabelValueLabel);
        m_rightPane.add(TCTLabelValueLabel);

//init
	ConModeBox.setSelectedIndexNA(ALL_CONGRUENCES);
	SetMode(ALL_CONGRUENCES, -1, true, -1);

    }

////////////////////////////////
//End of constructor

    public int getSelectedCongruence() {
	return(m_selected);
    }

    public int getSelectedCover() {
	return(m_selected2);
    }

    public boolean getUpperCoversAreShown() {
	return(m_UpperCoversAreShown);
    }


    public void setSelectedCongruence(int mode, int cong) {
	//<code>cong</code> is the index in the list
	//and not a congruence number!
	//If mode = ALL_CONGRUENCES, these are the same, otherwise not.

	ConModeBox.setSelectedIndexNA(mode);
	SetMode(mode, cong, m_UpperCoversAreShown, -1);
    }

    public void setSelectedCover(int low, int high) {

	ConModeBox.setSelectedIndexNA(ALL_CONGRUENCES);
	int cong, cover;
	if( m_UpperCoversAreShown) {
	    cong = low;
	    cover = high;
	} else {
	    cong = high;
	    cover = low;
	}
	SetMode(ALL_CONGRUENCES, cong, m_UpperCoversAreShown, cover);
	ConListBox.setSelectedIndexNA(cong);
    }

    public void setSelectedPrincipal(int [] gen) {
	if(gen[0] == gen[1] ) {
	    setSelectedCongruence(ALL_CONGRUENCES, zeroCongNum);
	    return;
	}
	int a, b;
	if( gen[0] > gen [1] ) {a = gen[1]; b = gen[0];}
	else {a = gen[0]; b = gen[1];}
	int index  = a*(algSize)+b - (a+1)*(a+2)/2;
	setSelectedCongruence(PRINCIPAL_CONGRUENCES, index);
    }

    public void displayTCTlabel() {
	//Display the TCT type of this covering.
	int low, high;
	if( m_UpperCoversAreShown ) {
	    low = m_selected;
	    high = m_selected2;
	} else { //Lower covers are shown
	    high = m_selected;
	    low = m_selected2;
	}
	int typ = m_congs.getTCTtype(low, high);
	String s = "typ(C" + low + ", C" + high + ") ";
	if( typ <= 0 ) {
	    s += "is unknown";
	} else {
	    s += "= " + typ;
	}
	TCTLabelValueLabel.setText(s);
    }

///////////////////////////
//Manage congruence display


    private void SetMode(int sel, int cong, boolean upper, int cover) {
	//upper is whether upper or lower covers should be shown.
	//cover is the congruence number of the cover to be shown.
	//
	//cong is the index of the congruence in the box 
	//corresponding to sel, and not a congruence number!
	//if cong == -1, we select the first "sensible" element:
	//  for sel == ALL_CONGRUENCES we set cong = intervalBottom,
	//  otherwise the first element in [intervalBottom, intervalTop].
	//
	//cover is a congruence number, not an index!
	//If cover == -1, then the first cover in 
	//[intervalBottom, intervalTop] is shown.

	m_mode = sel;
	switch( m_mode ) {
	    case ALL_CONGRUENCES: 
		ConListBox.setModelNA(ConListModel);
		break;
	    case MEET_IRRED_CONGRUENCES: 
		ConListBox.setModelNA(MIListModel);
		break;
	    case JOIN_IRRED_CONGRUENCES: 
		ConListBox.setModelNA(JIListModel);
		break;
	    case PRINCIPAL_CONGRUENCES: 
		ConListBox.setModelNA(PrinListModel);
		break;
	    default:
		return;
	}
	colorSelectedSubset();
	SetCurrentCongruence(cong, upper, cover);
    }


    private void SetCurrentCongruence(int cong, boolean upper, int cover) {
	//See SetMode() for the meaning of parameters.

	int i;
	int index;
	switch( m_mode ) {
	    case ALL_CONGRUENCES: //All congruences are listed
		if( cong == -1 ) {
		    m_selected = intervalBottom;
		} else {
		    m_selected = cong;
		}
		ConListBox.setSelectedIndexNA(m_selected);
		break;
	    case MEET_IRRED_CONGRUENCES: //Meet-irreducibles are listed
		index = 0;
		if( cong == -1 ) {
		    for( i=0; i< mi.length; i++) {
			if( isInInterval(mi[i]) ) {
			    index = i;
			    break;
			}
		    }
		} else {
		    index = cong;
		}
		m_selected = mi[index];
		ConListBox.setSelectedIndexNA(index);
		break;
	    case JOIN_IRRED_CONGRUENCES: //Join irreducibles are listed
		index = 0;
		if( cong == -1 ) {
		    for( i=0; i< ji.length; i++) {
			if( isInInterval(ji[i]) ) {
			    index = i;
			    break;
			}
		    }
		} else {
		    index = cong;
		}
		m_selected = ji[index];
		ConListBox.setSelectedIndexNA(index);
		break;
	    case PRINCIPAL_CONGRUENCES: //Principals are listed
		index = 0;
		if( cong == -1 ) {
		    for( i=0; i< prin.length; i++) {
			if( isInInterval(prin[i]) ) {
			    index = i;
			    break;
			}
		    }
		} else {
		    index = cong;
		}
		m_selected = prin[index];
		ConListBox.setSelectedIndexNA(index);
		break;
	    default:
		return;
	}

	//Now m_selected is the real congruence number
	//Display Human partition
	Partition part = m_congs.getPartition(m_selected);
	PartitionValueLabel.setText("C" + m_selected + " is " +
	    part.toString(Partition.BLOCK));


	Vector UpDownMode = new Vector();

	//Set up the covers.

	if( m_selected == zeroCongNum ) {
	    //Zero congruence, only upper covers exist.
	    UpDownMode.add(m_congs.getUpperCovers(m_selected).length +
			   " upper cover(s) of \"zero\" (see on the right)" );
	    UpDownModeBox.setModelNA(new DefaultComboBoxModel(UpDownMode));
	    SetUpDownMode(UPPER_COVERS, cover);//First argument is irrelevant and
	    //the selection is set correctly, since there is only one entry
	    return;
	} 
	if ( m_selected == oneCongNum ) {
	    //One congruence, only lower covers exist.
	    UpDownMode.add(m_congs.getLowerCovers(m_selected).length +
			   " lower cover(s) of \"one\" (see on the right)" );
	    UpDownModeBox.setModelNA(new DefaultComboBoxModel(UpDownMode));
	    SetUpDownMode(LOWER_COVERS, cover);//First argument is irrelevant and
	    //the selection is set correctly, since there is only one entry
	    return;
	}
	UpDownMode.add(m_congs.getUpperCovers(m_selected).length +
		       " upper cover(s) of C" + m_selected + 
		       " (see on the right)" );
	UpDownMode.add(m_congs.getLowerCovers(m_selected).length +
		       " lower cover(s) of C" + m_selected + 
		       " (see on the right)" );
	UpDownModeBox.setModelNA(new DefaultComboBoxModel(UpDownMode));
	//This sets the index first to zero.
	if( upper ) {
	    SetUpDownMode(UPPER_COVERS, cover);
	    UpDownModeBox.setSelectedIndexNA(UPPER_COVERS);
	    return;
	}
	UpDownModeBox.setSelectedIndexNA(LOWER_COVERS);
	SetUpDownMode(LOWER_COVERS, cover);
    }

    private void SetUpDownMode(int sel, int cover) {
	if( m_selected == zeroCongNum ||
	    (( m_selected != oneCongNum ) && sel == UPPER_COVERS) ) {
	    fillUpperCovers(m_selected, CoverListBox);
	    m_UpperCoversAreShown = true;
	} else {
	    fillLowerCovers(m_selected, CoverListBox);
	    m_UpperCoversAreShown = false;
	}
	//Selection in CoverListBox is already set to 0
	//because of the model changes, but we do not use this.
	int i;
	int index;
	if(cover == -1) {
	    index = 0;
	    for( i=0; i< covNums.length; i++) {
		if( isInInterval(covNums[i]) ) {
		    index = i;
		    break;
		}
	    }
	} else {
	    index = getIndexOfCover(cover);
	}
	CoverListBox.setSelectedIndexNA(index);
	SetCurrentCongruence2(index);
    }

    private void SetCurrentCongruence2(int sel) {
	//This can be called ONLY if lower or upper
	//covers are set up properly by SetUpDownMode()
	//sel is the index.

	m_selected2 = covNums[sel];

	//Display Human partition
	Partition part = m_congs.getPartition(m_selected2);
	Partition2ValueLabel.setText("C" + m_selected2 + " is " +
	    part.toString(Partition.BLOCK));

	displayTCTlabel();
        // rsf new
	highlightCover();
    }

    private int getIndexOfCover(int cover) {
	Integer i = (Integer)covMap.get(new Integer(cover));
	if(i == null) return -1;//not a cover
	return(i.intValue());
    }

    private void fillUpperCovers(int c, ewkComboBox box) {

	Vector covStrings = new Vector();
	covNums = m_congs.getUpperCovers(c);
	covMap = new HashMap(covNums.length);
	int i;
	for( i=0; i<covNums.length; i++ ) {
	    covMap.put(new Integer(covNums[i]), new Integer(i));
	    covStrings.add(ConList.elementAt(covNums[i]));
	}
	box.setModelNA(new DefaultComboBoxModel(covStrings));
	//This sets selected index to 0 automatically!
    }

    private void fillLowerCovers(int c, ewkComboBox box) {

	Vector covStrings = new Vector();
	covNums = m_congs.getLowerCovers(c);
	covMap = new HashMap(covNums.length);
	int i;
	for( i=0; i<covNums.length; i++ ) {
	    covMap.put(new Integer(covNums[i]), new Integer(i));
	    covStrings.add(ConList.elementAt(covNums[i]));
	}
	box.setModelNA(new DefaultComboBoxModel(covStrings));
	//This sets selected index to 0 automatically!
    }

////////////////////////////////
// METHODS involving the draw window. Note all check that latdraw != null
//         so they're safe.

    /**
     * Open a lattice drawer window.
     */
    public void setLatticeDrawer(Project fn_Project) {

	if( latdraw != null ) return; //just for insurance
	latdraw = new LatticeDisplayer2(this, fn_Project);
	latdraw.setTCTColorMap(
	    Configure.type1Color,
	    Configure.type2Color,
	    Configure.type3Color,
	    Configure.type4Color,
	    Configure.type5Color);
	latdraw.setBackgroundColor(Configure.backgroundColor);
	setIntervalValueLabelText();
        highlightCover();

        latdraw.addPropertyChangeListener(new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
		    String prop = e.getPropertyName();
		    if ("VertexClicked".equals(prop) || 
			"VertexRightClicked".equals(prop)) {
			String label = (String)e.getNewValue();
			int cong = CongruenceLattice.drawerLabelToInt(label);
			if(cong < 0 || cong > m_congs.getSize() ) return;
			setSelectedCongruence(
			    CongruenceLatticeDisplayer.ALL_CONGRUENCES, cong);
		    }
		    if ("EdgeClicked".equals(prop) || 
			"EdgeRightClicked".equals(prop)) {
			String[] labels = (String[])e.getNewValue();
			int low = CongruenceLattice.drawerLabelToInt(labels[0]);
			if(low < 0 || low > m_congs.getSize() ) return;
			int high = CongruenceLattice.drawerLabelToInt(labels[1]);
			if(high < 0 || high > m_congs.getSize() ) return;
			setSelectedCover(low, high);
		    }
		    if ("DiagramChanged".equals(prop)) {
                        Vector v = (Vector)e.getNewValue();
                        drawingInterval 
                                 = ((Boolean)v.elementAt(0)).booleanValue();
                        intervalBottom = CongruenceLattice.drawerLabelToInt(
                                                      (String)v.elementAt(1));
                        intervalTop = CongruenceLattice.drawerLabelToInt(
                                                      (String)v.elementAt(2));
			//At the moment, we do not use v.elementAt(3)
			//indicating whether the interval is newly created.

			colorSelectedSubset();
			highlightCover();
			setIntervalValueLabelText();
		    }
		}
	    });
    }


    public void disposeDrawer() {
	if(latdraw != null) {
	    if( latdraw.isEnabled()) latdraw.dispose();
	}
	latdraw = null;
	setIntervalValueLabelText();
        intervalBottom = zeroCongNum;
        intervalTop = oneCongNum;
	drawingInterval = false;
	m_parent.OnDrawerClosed();
    }

    public void unHighlightCover() {
      if (latdraw != null) {
        latdraw.clearHighlightedEdges();
        unColorAllElementBorders();
      }
    }

    public void highlightCover() {
        if ( !highlightingOn ) return;
        if (latdraw == null) return;
	unHighlightCover();
	
	int low = getSelectedCongruence();
	if( !isInInterval(low) ) return;
	//This may change in the future: we might
	//highlight the covers that are in the interval
	//even when the selected point is outside.
	
	colorElementBorder(low, Configure.selectedBorderColor, true);
	
	int i;
	int cong;
	for(i=0; i<covNums.length; i++) {
	    cong = covNums[i];
	    if( !isInInterval(cong) ) continue;
	    colorElementBorder(cong, Configure.coverBorderColor, false);
	}
	
	int high = getSelectedCover();
	if( !isInInterval(high) ) return;
	colorElementBorder(high, Configure.coverBorderColor, true);
	
	if (!m_UpperCoversAreShown) {
	    int tmp = low;
	    low = high;
	    high = tmp;
	}
	latdraw.addHighlightedEdge(
	    new String[] {drawerLabels[low], drawerLabels[high]});
    }
    
    public void colorElement(int elem, Color color) {
	if (latdraw != null) {
	    latdraw.addColoredElement(drawerLabels[elem], color);
	}
    }

    public void unColorElement(int elem) {
	if (latdraw != null) {
	    latdraw.removeColoredElement(drawerLabels[elem]);
	}
    }

    public void unColorAllElements() {
	if (latdraw != null) {
	    latdraw.clearColoredElements();
	}
    }

    public void colorElementBorder(int elem, Color color, boolean thick) {
	if (latdraw != null) {
	    latdraw.addBorderColoredElement(drawerLabels[elem], color, thick);
	}
    }
    
    public void unColorElementBorder(int elem) {
	if (latdraw != null) {
	    latdraw.removeBorderColoredElement(drawerLabels[elem]);
	}
    }
    
    public void unColorAllElementBorders() {
	if (latdraw != null) {
	    latdraw.clearBorderColoredElements();
	}
    }

// Next 4 functions: Color selected subset of vertices:

    private void colorSelectedSubset() {
	if( !highlightingOn ) return;
	if ( drawingInterval ) {
	    unColorAllElements();
	    return;
	}
	switch( m_mode ) {
	    case ALL_CONGRUENCES: 
                unColorAllElements();
		break;
	    case MEET_IRRED_CONGRUENCES: 
                colorMIs();
		break;
	    case JOIN_IRRED_CONGRUENCES: 
                colorJIs();
		break;
	    case PRINCIPAL_CONGRUENCES: 
                colorPrincipals();
		break;
	    default:
		return;
	}
    }


    public void colorJIs() {
	unColorAllElements();
	for (int i = 0; i < ji.length; i++) {
	    colorElement(ji[i], Configure.jiColor);
	}
    }

    public void colorMIs() {
	unColorAllElements();
	for (int i = 0; i < mi.length; i++) {
	    colorElement(mi[i], Configure.miColor);
	}
    }

    public void colorPrincipals() {
	unColorAllElements();
	for (int i = 0; i < prinsAsSet.length; i++) {
	    colorElement(prinsAsSet[i], Configure.prinColor);
	}
    }

//////////////////////////////////////////////////

    private void setIntervalValueLabelText() {
	if( latdraw == null ) {
	    IntervalValueLabel.setText(
		"None");
	    return;
	}
	if( !drawingInterval ) {
	    IntervalValueLabel.setText(
		"Entire lattice");
	} else {
	    IntervalValueLabel.setText(
		"[" + intervalBottom + ", " + intervalTop + "]");
	}
    }

    private boolean isInInterval(int cong) {
	if( !m_congs.isLEQ(cong, intervalTop) ) return false;
	if( !m_congs.isLEQ(intervalBottom, cong) ) return false;
	return true;
    }

//Components
    
    private LatticeDisplayer2 latdraw = null;

    // This boolean could be used to turn highlighting on and off.
    // Right now it is always on.
    private boolean highlightingOn = true;  // rsf

    private CongruenceDisplayerFrame m_parent;
    private JPanel m_leftPane;
    private JPanel m_rightPane;
    private JLabel IntervalTextLabel;
    private JLabel IntervalValueLabel;
    private JLabel PartitionTextLabel;
    private JLabel PartitionValueLabel;
    private ewkComboBox ConModeBox;
    private ewkComboBox ConListBox;
    private JLabel Partition2TextLabel;
    private JLabel Partition2ValueLabel;
    private ewkComboBox UpDownModeBox;
    private ewkComboBox CoverListBox;
    private JLabel TCTLabelTextLabel;
    private JLabel TCTLabelValueLabel;

    private Algebra m_A;
    private int algSize;
    private CongruenceLattice m_congs;
    private Vector ConList;
    private DefaultComboBoxModel ConListModel;
    private DefaultComboBoxModel MIListModel;
    private DefaultComboBoxModel JIListModel;
    private DefaultComboBoxModel PrinListModel;
    private int[] covNums;

    // rsf 
    private String[] drawerLabels;

    private HashMap covMap;
    private int[] mi;
    private int[] ji;
    private int[] prin;
    private int[] prinsAsSet; // rsf
    private int m_selected = 0;
    private int m_selected2;
    private int m_CoverListBoxSelected2;
    private int m_mode;
    private boolean m_UpperCoversAreShown;
    private int zeroCongNum;
    private int oneCongNum;

//Selection modes
    public static final int ALL_CONGRUENCES = 0;
    public static final int MEET_IRRED_CONGRUENCES = 1;
    public static final int JOIN_IRRED_CONGRUENCES = 2;
    public static final int PRINCIPAL_CONGRUENCES = 3;

    public static final int UPPER_COVERS = 0;
    public static final int LOWER_COVERS = 1;

// Drawer State
    private boolean drawingInterval;
    private int intervalBottom;
    private int intervalTop;

}


abstract class ewkComboBox extends JComboBox {
/**
 * Has an abstract userSelected() method, which
 * is called, when the user makes a NEW selection.
 * That is, it is NOT called when the user makes a selection
 * but it is the same as the old selection.
 * We also provide methods for setting the Model
 * and the selectedIndex but do not fire action change.
 */

    public ewkComboBox() {

	super();
	addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    int sel = getSelectedIndex();
		    if( sel == lastSelection ) return;
		    lastSelection = sel;
		    if( !actionEnabled ) return;
		    userSelected();
		}
	    }); 
    
    }

    public void setModelNA(ComboBoxModel aModel) {
	actionEnabled = false;
	super.setModel(aModel);
	actionEnabled = true;
    }

    public void setSelectedIndexNA(int index) {
	actionEnabled = false;
	setSelectedIndex(index);
	actionEnabled = true;
    }

    public abstract void userSelected();

    private int lastSelection = -1;
    //we assume that the default constructor sets the
    //selected index to -1.

    private boolean actionEnabled = true;

}

