package uacalc.alg.opedit;

import uacalc.alg.Operation;
import uacalc.alg.OpArguments;

import uacalc.ex.TooManyArgumentsException;

import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;

public class OperationRendererDirect
    extends OperationRenderer {

    public OperationRendererDirect(Operation f,
				   ValueEditor editor,
				   Container pane) {
	super(f, editor, pane);
	m_ValueColumn = ValueEditorList.ValueColumn;

	try {
	    currentArgs = new OpArguments(m_arity);
	} catch(TooManyArgumentsException e) {
	    System.out.println(e.toString());
	    return;
	}
	int i;
	for(i=0; i<m_arity; i++) {
	    currentArgs.setValue(i, 0);
	}

	directModel = new OpValueDirectModel();
	OpValueDirect = 
	    new ValueEditorList(m_f, directModel);

	OpValueDirect.setTableCursorAt(0);

	m_pane.setLayout(new BoxLayout(m_pane, BoxLayout.X_AXIS));
	m_pane.add(OpValueDirect);

    }

    public void editValueAt(OpArguments args, int value) {
	currentArgs = args;
	directModel.fireTableDataChanged();
	OpValueDirect.editCellAt(0, value);
    }

    public void editValueAt(OpArguments args) {
	currentArgs = args;
	directModel.fireTableDataChanged();
	OpValueDirect.editCellAt(0, -2);
    }

    public void Update() {
	directModel.fireTableDataChanged();
	OpValueDirect.setFocusToEditedCell();
    }

    public boolean isEditorOpen() {
	return OpValueDirect.isEditorOpen();
    }

    private int m_ValueColumn;
    private ValueEditorList OpValueDirect;
    private OpValueDirectModel directModel;
    protected OpArguments currentArgs;

//inner classes

    class OpValueDirectModel extends AbstractTableModel {
    
	public int getColumnCount() {
	    return 2;
	}
    
	public int getRowCount() {
	    return 1;
	}
    
	public Object getValueAt(int row, int col) {
	    
	    if(col == m_ValueColumn) {
		return new Integer(
		    m_f.getValue(currentArgs));
	    }
	    String s = new String(" = " + currentArgs.toString(m_f));
	    //since m_ValueColumn = 0
	    return s;
	}
    
	public void setValueAt(Object value, int row, int col) {
	    //Cannot set String column at all
	    
	    if(col == m_ValueColumn) {
		int intval = ((Integer)value).intValue();
		if( !m_f.tryValue(currentArgs, intval )) {
		    OnInvalidValue( currentArgs, intval );
		}
		fireTableCellUpdated(row, col);
		OperationChanged();
	    }
	}
    
    
	public Class getColumnClass(int col) {
	    if(col == m_ValueColumn ) return Integer.class;
	    return String.class;
	}
	
	public boolean isCellEditable(int row, int col) {
	    if(col == m_ValueColumn) return true;
	    return false;
	}
	
    }
}

