package uacalc.alg.opedit;

import uacalc.alg.Operation;
import uacalc.alg.OpArguments;
import uacalc.alg.algedit.AlgebraEditor;
import uacalc.vectors.VectorInput;

import uacalc.ex.TooManyArgumentsException;

import java.awt.*;
import java.io.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;

public class ValueEditor {

    public ValueEditor(AlgebraEditor parent,
		        Operation f) {

//initialize local variables
	m_AlgebraEditor = parent;

	m_f = f;
	m_size = f.getSize();
	m_arity = f.getArity();

	MainFrame = new JFrame();

//Set up menus
        JMenuBar menuBar = new JMenuBar();
        MainFrame.setJMenuBar(menuBar);

//File menu
        JMenu FileMenu = new JMenu("File");
	FileMenu.setMnemonic(KeyEvent.VK_F);
        menuBar.add(FileMenu);

//Save
        SaveMenuItem = new JMenuItem("Save", KeyEvent.VK_S);
	FileMenu.add(SaveMenuItem);
	SaveMenuItem.setAccelerator(
	    KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));

	SaveMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    OnSave();
		}
	    });

//Close
        CloseMenuItem = new JMenuItem("Close", KeyEvent.VK_C);
	FileMenu.add(CloseMenuItem);
	CloseMenuItem.setAccelerator(
	    KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK));

	CloseMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    OnClose();
		}
	    });


	if( m_arity >= 1 ) {
//Arguments menu
	    JMenu ArgumentsMenu = new JMenu("Arguments");
	    ArgumentsMenu.setMnemonic(KeyEvent.VK_A);
	    menuBar.add(ArgumentsMenu);


//Select
	    SelectMenuItem = new JMenuItem("Select...", KeyEvent.VK_E);
	    ArgumentsMenu.add(SelectMenuItem);
	    SelectMenuItem.setAccelerator(
		KeyStroke.getKeyStroke(KeyEvent.VK_E, ActionEvent.CTRL_MASK));

	    SelectMenuItem.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			OnSelect();
		    }
		});
	}

//Help menu
        JMenu HelpMenu = new JMenu("Help");
	HelpMenu.setMnemonic(KeyEvent.VK_H);
        menuBar.add(HelpMenu);


//Tips
        TipsMenuItem = new JMenuItem("Tips...", KeyEvent.VK_P);
	HelpMenu.add(TipsMenuItem);
	TipsMenuItem.setAccelerator(
	    KeyStroke.getKeyStroke(KeyEvent.VK_P, ActionEvent.CTRL_MASK));

	TipsMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    OnTips();
		}
	    });


	pane = MainFrame.getContentPane();
	pane.setLayout(new BoxLayout(pane, BoxLayout.X_AXIS));

	try {
	    args = new OpArguments(m_arity);
	} catch(TooManyArgumentsException e) {
	    System.out.println(e.toString());
	    return;
	}
	int i;
	for(i=0; i<m_arity; i++) {
	    args.setValue(i, 0);
	}

	paneDirect = new JPanel();
	directRenderer = new OperationRendererDirect(
	    m_f, this, paneDirect);
	tabbedPane = new JTabbedPane();
	tabbedPane.addTab("Direct", paneDirect);
	renderers = new OperationRenderer[3];
	renderers[DIRECT] = directRenderer;
	editorType = DIRECT;

	if( m_arity >= 1 ) {
	    paneList = new JPanel();
	    listRenderer = new OperationRendererList(
	    m_f, this, paneList);
	    tabbedPane.addTab("List", paneList);
	    renderers[LIST] = listRenderer;
	    editorType = LIST;
	}

	if( m_arity == 2 ) {
	    paneTable = new JPanel();
	    tableRenderer = new OperationRendererTable(
	    m_f, this, paneTable);
	    tabbedPane.addTab("Table", paneTable);
	    renderers[TABLE] = tableRenderer;
	    editorType = TABLE;
	}
	tabbedPane.setSelectedIndex(editorType);

	pane.add(tabbedPane);
	tabbedPane.addChangeListener(new ChangeListener() {
                public void stateChanged(ChangeEvent e) {
                    OnTabChanged();
                }
            });


	MainFrame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        MainFrame.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    OnClose();
                }
		public void windowActivated(WindowEvent e) {
                    OnTabChanged();
		    //?????????? Is it a good idea to update data
		    //on every loss of focus???????????
                }             
	    });

	SaveMenuItem.setEnabled(false);
        MainFrame.setTitle("Value Editor");
	MainFrame.pack();
	parent.setEnabled(false);
        MainFrame.setVisible(true);
    }



////////////////////////////////////
//Menu callbacks:
////////////////////////////////////

//File menu callbacks

    public void OnSave() {

	if( !OnEditorOpen() ) return;
	m_AlgebraEditor.OnSave();
	SaveMenuItem.setEnabled(false);
    }


    public void OnClose() {

	if( !OnEditorOpen() ) return;
	m_AlgebraEditor.setEnabled(true);
	m_AlgebraEditor.requestFocus();
	MainFrame.dispose();
    }


    public void OnTabChanged() {

	editorType = tabbedPane.getSelectedIndex();
	renderers[editorType].Update();
    }

    public void OnSelect() {

	int[] sizes = new int[args.getLength()];
	int i;
	for( i=0; i< args.getLength(); i++ ) {
	    sizes[i]=m_size;
	}
	VectorInput dialog = new VectorInput(
	    MainFrame, 
	    args.getValues(),
	    sizes,
	    "Select arguments",
	    "Enter the list of arguments:");
	if ( !dialog.isCancelled() ) {
	    renderers[editorType].editValueAt(args);
	}
    }

    public void OnTips() {
	JOptionPane.showMessageDialog(
	    MainFrame,
	    "To edit a cell:\n" +
	    "    double click on it with the mouse, OR\n" +
	    "    move there with the arrows, TAB or Enter,\n" +
	    "        and then press F2.\n" +
	    "Another double click selects the entire text.\n" +
	    "Press Enter to set the new value in the cell;\n" +
	    "Press Esc to cancel editing the cell.",
	    "Editing tips",
	    JOptionPane.INFORMATION_MESSAGE);
    }

    private boolean OnEditorOpen() {
	//Returns false to cancel.

	int count = 0;
	int i;
	for( i=0; i<3; i++ ) {
	    if( renderers[i] != null ) {
		if( renderers[i].isEditorOpen() ) count ++;
	    }
	}
	if(count == 0) return true;
	int ans = JOptionPane.showConfirmDialog(
	    MainFrame,
	    "An entry is still being edited in " +
	    count + " cell(s).\n" +
	    "Please remember: you have to press Enter, TAB, or an arrow key,\n" +
	    "or click on another cell after editing to activate the change.\n" +
	    "Do you want to abandon the changes made in the " +
	    count + " edited cell(s)?",
	    "Abandon changes",
	    JOptionPane.YES_NO_OPTION,
	    JOptionPane.WARNING_MESSAGE);

	if(ans != JOptionPane.YES_OPTION ) return(false);
	return true;
    }

    public void enableSaveMenu() {
	m_AlgebraEditor.setAlgebraModified(true);
	SaveMenuItem.setEnabled(true);
    }

//private variables

    JFrame MainFrame;
    private AlgebraEditor m_AlgebraEditor;
    private int m_size;
    private Operation m_f;
    private int m_arity;
    private int editorType;
    public static final int DIRECT = 0;
    public static final int LIST = 1;
    public static final int TABLE = 2;

    private OpArguments args;

//MenuItems
    private JMenuItem SaveMenuItem;
    private JMenuItem CloseMenuItem;

    private JMenuItem SelectMenuItem;

    private JMenuItem TipsMenuItem;

//Components
    private Container pane;
    private OperationRendererDirect directRenderer = null;
    private OperationRendererList listRenderer = null;
    private OperationRendererTable tableRenderer = null;
    private OperationRenderer renderers[];

    private JTabbedPane tabbedPane;
    JPanel paneDirect;
    JPanel paneList;
    JPanel paneTable;

}

