package uacalc.util;

import java.awt.event.*;
import javax.swing.*; 
import javax.swing.text.*; 

import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;

public class WholeNumberField extends JTextField {
    private NumberFormat integerFormatter;

    public WholeNumberField(int value, int columns) {
        super(columns);
        integerFormatter = NumberFormat.getNumberInstance(Locale.US);
        integerFormatter.setParseIntegerOnly(true);
	setHorizontalAlignment(WholeNumberField.RIGHT);
        setValue(value);
    }

    public int getValue() {
        int retVal = 0;
        try {
            retVal = integerFormatter.parse(getText()).intValue();
        } catch (ParseException e) {
	    //This means that the string is empty
	    retVal= -1;
        }
        return retVal;
    }

    public void setValue(int value) {
	if(value == -1) setText("");
        else setText(integerFormatter.format(value));
    }

    protected Document createDefaultModel() {
        return new WholeNumberDocument();
    }

    protected class WholeNumberDocument extends PlainDocument {
        public void insertString(int offs, 
                                 String str,
                                 AttributeSet a) 
                throws BadLocationException {
            char[] source = str.toCharArray();
            char[] result = new char[source.length];
            int j = 0;

            for (int i = 0; i < result.length; i++) {
                if (Character.isDigit(source[i]))
                    result[j++] = source[i];
            }
            super.insertString(offs, new String(result, 0, j), a);
        }
    }
}
