package uacalc.util;

import java.awt.*;
import java.io.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;

/**
 * Allows the user to select a file name in a customized way
 * using JFileChooser.
 */

public class ewkFileChooser {

    public ewkFileChooser() {
	m_fc = new JFileChooser(System.getProperty("user.dir"));
    }

/**
 * title: title of the filechooser
 * dtype: one of OPEN, NEW, DELETE.
 * Returns null on Cancel.
 * Allows the user to select an existing file for NEW and SAVE_AS,
 * but disallows to select a non-existing one for OPEN and DELETE.
 */

    public static String getFile(
	JFrame parent,
	SimpleFileFilter filter,
	String title,
	int dtype) {

	switch( dtype ) {
	    case OPEN:
		m_fc.setApproveButtonMnemonic(KeyEvent.VK_O);
		m_fc.setApproveButtonToolTipText("Open selected file");
		m_fc.setApproveButtonText("Open");
		break;
	    case DELETE:
		m_fc.setApproveButtonMnemonic(KeyEvent.VK_D);
		m_fc.setApproveButtonToolTipText("Delete selected file");
		m_fc.setApproveButtonText("Delete");
		break;
	    case NEW:
		m_fc.setApproveButtonMnemonic(KeyEvent.VK_N);
		m_fc.setApproveButtonToolTipText("Accept name of new file");
		m_fc.setApproveButtonText("New");
		break;
	    case SAVE_AS:
		m_fc.setApproveButtonMnemonic(KeyEvent.VK_V);
		m_fc.setApproveButtonToolTipText("Accept name of new file");
		m_fc.setApproveButtonText("Save As");
		break;
	    default:
		System.out.println("Invalid FileChooser dtype.");
	}


	if(title != null ) {
	    m_fc.setDialogTitle(title);
	} else {
	    switch( dtype ) {
		case OPEN:
		    m_fc.setDialogTitle("Open");
		    break;
		case DELETE:
		    m_fc.setDialogTitle("Delete");
		    break;
		case NEW:
		    m_fc.setDialogTitle("New");
		    break;
		case SAVE_AS:
		    m_fc.setDialogTitle("Save As");
		    break;
		default:
		    System.out.println("Invalid FileChooser dtype.");
			}
	}

	m_fc.setFileFilter(filter);
	    
	int returnVal;
	String fn;
	while(true) {
	    returnVal = m_fc.showDialog(parent, null);
	    if(returnVal != JFileChooser.APPROVE_OPTION) {
		return(null);
	    }
	    File file = m_fc.getSelectedFile();
	    fn = validateName(file, parent, filter);
	    if( fn == null ) continue;//User cancelled validation
	    file = new File(fn);
	    boolean exists = file.exists();
	    if( exists || dtype == NEW || dtype == SAVE_AS) {
		return(fn);
	    }
	    JOptionPane.showMessageDialog(
		parent, 
		"Please select an existing file.",
		"File does not exist",
		JOptionPane.ERROR_MESSAGE);
	    continue;
	}
    }

    private static String validateName(File file, Frame parent,
				SimpleFileFilter filter) {
    //Returns null on cancel, new name otherwise.

	String fn = file.getPath();
	String extension = filter.getExtension();
	if( !filter.accept(file) ) {
	    int ans = JOptionPane.showConfirmDialog(
		parent, 
		"File must have extension \""
		+ extension + "\".\n"
		+"Do you want this extension to be appended?",
		"Extension Error",
		JOptionPane.YES_NO_OPTION,
		JOptionPane.WARNING_MESSAGE);
	    if(ans == JOptionPane.YES_OPTION) {
		return(FileNameExtension.appendExtension(fn,extension));
	    } else return(null);
	}
	return(fn);
    }

    private static JFileChooser m_fc;
    public static final int OPEN = 1;
    public static final int DELETE = 2;
    public static final int NEW = 3;
    public static final int SAVE_AS = 4;
}
