package uacalc.util;

//Adds get_integer method to BufferedInputStream.

import java.lang.*;
import java.io.*;

public class ewkInputStream extends BufferedInputStream {

    public ewkInputStream(InputStream in) {
	super(in);
    }

    public boolean isEOF() {
	//This will discard the next character of the file
	//if NOT EOF, so call this only for error checking.
	try {
	    if( read() == -1 ) {
		return(true);
	    }
	}
	catch(IOException e) {
	    return(true);
	}
	return(false);
    }

    public int skipTo(String delim) {
	//Read the file until you encounter a character in delim.
	//Discard as you go.
	//Return the delimiter found, or -1 on EOF.

	int ch;
	try {
	    while (true ) {
		ch = read();
		if( ch == -1 || delim.indexOf(ch) != -1 ) return(ch);
	    }
	}
	catch(IOException e) {
	    return(-1);
	}
    }

    public int get_integer(String delim) {

// Reads the next non-negative integer.  That is, it reads characters
// as long as they are digits, but less than NUM_LEN_LIMIT
// characters. Skips starting whitespace.  If the digits are followed
// by a character in delim, the integer corresponding to these digits
// is returned.  Otherwise, or on error, -1 is returned.  EOF cannot
// be in delim, but it is always considered as a delimiter.  Returns
// -2 if there are no more integers to read because of EOF.  The file
// pointer points to the first character after the delimiter, if there
// was no error.
	
	int ch, count, num;
	int sr;
	
	StringBuffer st = new StringBuffer(0);
	try {
	    for(;;) {
		ch = read();
//End of file, after skipping whitespace.
		if(ch == -1) return(-2);
		if(ch == ' ') continue;
		if(ch == '\n') continue;
		if(ch == '\r') continue;
		if(ch == '\t') continue;
		if(isdigit(ch)) break;
		return(-1);
	    }
	    for(count=0; count<NUM_LEN_LIMIT-1; count++) {
		st.append((char)ch);
		ch = read();
		if(ch == -1) break;
//No delimiter needed at the end of the file.
		if(!isdigit(ch)) break;
	    }
	    if(count>=NUM_LEN_LIMIT-1) return(-1);
	    /* NUM_LEN_LIMIT digits read */
	    if( ch != -1 && delim.indexOf(ch) == -1 ) return(-1);
	}
	catch(IOException e) {
	    return(-1);
	}
// OK, parse number
	try {
	    num=Integer.parseInt(st.toString());
	} 
	catch(NumberFormatException e) {
	    return(-1);
	}
	return(num);
    }

    private boolean isdigit(int ch) {
//We need this to avoid the UNICODE stuff
//which is unnecessary here, as we want to keep the old file format
	if( 48 <= ch && ch <=57 ) return(true);
	return(false);
    }

    public static final int NUM_LEN_LIMIT = 20;
//When we align numbers on output, we do not want to print
//NUM_LEN_LIMIT many spaces.
    public static final int NUM_LEN_OUT_LIMIT = 6;
}
