package uacalc.vectors;

import uacalc.alg.con.Partition;
import uacalc.alg.con.CongruenceLattice;
import uacalc.util.Project;
import uacalc.vectors.VectorList;

import java.util.*;
import java.io.*;

public class BinaryRelation extends VectorList {

    public BinaryRelation() {
	m_length = 2;
    }

    public BinaryRelation(Partition part) {
	m_length = 2;
	m_partition = part;
    }

    public BinaryRelation(CongruenceLattice congs,
			  int CongruenceNumber) {
	m_length = 2;
	m_congs = congs;
	m_CongruenceNumber = CongruenceNumber;
    }

    public void importPartition(Partition part) {
	int size = part.getSize();
	int i, j;
	int[] next;
	for(i=0; i<size; i++) {
	    next = new int[2];
	    next[0] = i;
	    next[1] = i;
	    add(next);
	    for(j=i+1; j<size; j++) {
		if( part.isRelated(i, j) ) {
		    next = new int[2];
		    next[0] = i;
		    next[1] = j;
		    add(next);
		    next = new int[2];
		    next[0] = j;
		    next[1] = i;
		    add(next);
		}
	    }
	}
    }

    public String prepareBinRelLine(Project proj, int type) {
	if(m_CongruenceNumber != -1) {
	    return 
		"\\bin_rel{\\congruence{\\conlist_file{" +
		proj.getName(Project.ConListType) + 
		"}\\number{" + m_CongruenceNumber + "}}}";

	}
	if( m_partition != null) importPartition(m_partition);
	String name = proj.getName(type);

	try {
            FileOutputStream fout =
                new FileOutputStream(name);
            PrintStream fp =
                new PrintStream(fout);
            Write(fp, type);
            fp.close();
            fout.close();
        }
        catch(Exception e) {
	    System.out.println(e.toString());
	    return null;
        }

	return 
	    "\\bin_rel{\\vectorlist_file{" + name +
	    "}}";
    }

    public String toString() {
	if(m_CongruenceNumber != -1) {
	    return m_congs.getPartition(m_CongruenceNumber).
		toString(Partition.HUMAN);
	}
	if( m_partition != null) {
	    return m_partition.toString(Partition.HUMAN);
	}
	return "\n" + super.toString();
    }

    private Partition m_partition = null;
    private CongruenceLattice m_congs = null;
    private int m_CongruenceNumber = -1;

}
