package uacalc.vectors;

import uacalc.alg.Algebra;
import uacalc.config.Configure;
import uacalc.util.WholeNumberField;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.beans.*;

public class VectorInput extends JDialog {

    public VectorInput(Frame parent, int[] vector, int[] sizes,
		     String title, String prompt) {
//It is assumed that this is not nullary.
	
        super(parent, true);
	//This doesn't work for some reason, the dialog is not
	//centered over this frame. A partial solution is the next command:
	setLocationRelativeTo(parent);
    
	setTitle(title);
	m_vector = vector;
	m_length = vector.length;
	m_sizes = sizes;
	cancelled = false;

	JPanel pane = new JPanel();

	pane.setLayout(new BoxLayout(pane, BoxLayout.X_AXIS));

	NumberField = new WholeNumberField[m_length];
	int i;
	JLabel label;
	for(i = 0; i< m_length; i++) {
	    label = new JLabel(i+ ":");
	    label.setAlignmentX(Component.LEFT_ALIGNMENT);
	    Configure.createBorder(label);
	    label.setMinimumSize(
		label.getPreferredSize());
	    label.setMaximumSize(
		label.getPreferredSize());
	    pane.add(label);
	    NumberField[i] = new WholeNumberField(
		m_vector[i],
		Algebra.MAX_ALG_SIZE_DIGITS);
	    NumberField[i].setAlignmentX(Component.LEFT_ALIGNMENT);
	    NumberField[i].setMinimumSize(
		NumberField[i].getPreferredSize());
	    NumberField[i].setMaximumSize(
		NumberField[i].getPreferredSize());
	    pane.add(NumberField[i]);
	}
        Object[] array = {prompt,
			  pane
	};

        final String btnString1 = "Done";
        final String btnString2 = "Cancel";
        Object[] options = {btnString1, btnString2};

        optionPane = new JOptionPane(array, 
				     JOptionPane.QUESTION_MESSAGE,
				     JOptionPane.OK_CANCEL_OPTION,
				     null,
				     options,
				     options[0]);
        setContentPane(optionPane);
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
	optionPane.setValue(
	    JOptionPane.UNINITIALIZED_VALUE);

	focusWindow = 0;
        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent we) {
                /*
                 * Instead of directly closing the window,
                 * we're going to change the JOptionPane's
                 * value property.
                 */
                    optionPane.setValue(new Integer(
                                        JOptionPane.CLOSED_OPTION));
		}
		public void windowActivated(WindowEvent e) {
		    int j;
		    for(j=0; j< m_length; j++) {
			NumberField[j].selectAll();
		    }
		    NumberField[focusWindow].requestFocus();
		}
        });


	ActionListener ac = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    optionPane.setValue(btnString1);
		}
	    };
	for(i=0; i< m_length; i++) {
	    NumberField[i].addActionListener(ac);
	}

        optionPane.addPropertyChangeListener(new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent e) {
                String prop = e.getPropertyName();

                if (isVisible() 
                 && (e.getSource() == optionPane)
                 && (prop.equals(JOptionPane.VALUE_PROPERTY) ||
                     prop.equals(JOptionPane.INPUT_VALUE_PROPERTY))) {
                    Object v = optionPane.getValue();

                    if (v == JOptionPane.UNINITIALIZED_VALUE) {
                        //ignore reset
                        return;
                    }
                    // Reset the JOptionPane's value.
                    // If you don't do this, then if the user
                    // presses the same button next time, no
                    // property change event will be fired.
                    optionPane.setValue(
                            JOptionPane.UNINITIALIZED_VALUE);

                    if (v.equals(btnString1)) {
			validateArgs();
                    } else { 
			OnCancel();
                    }
                }
            }
        });
	pack();
        setVisible(true);
    }

    private void validateArgs() {
	int i;

	for(i=0; i<m_length; i++) {
	    int value = NumberField[i].getValue();
	    if (value < 0 || value >= m_sizes[i]) {
		// number was invalid
		JOptionPane.showMessageDialog(
		    VectorInput.this,
		    "Value " + i + " must be between " +
		    0 + " and " + (m_sizes[i]-1) + ".",
		    "Number out of bounds",
		    JOptionPane.ERROR_MESSAGE);
		focusWindow=i;
		return;
	    }
	}
//All values are valid
	for(i=0; i<m_length; i++) {
	    m_vector[i] = NumberField[i].getValue();
	    //We are done
	    cancelled = false;
	    setVisible(false);
	}
    }

    private void OnCancel() {
	cancelled = true;
	setVisible(false);
    }

    public boolean isCancelled() {
	return cancelled;
    }

    WholeNumberField [] NumberField;
    private int[] m_vector;
    private int[] m_sizes;
    private int m_length;
    private JLabel prompt;
    private JOptionPane optionPane;
    private int focusWindow;
    private boolean cancelled;
}

