//We do not catch OutOfMemory yet, it is for the future.

package uacalc.vectors;

import uacalc.util.ewkInputStream;
import uacalc.alg.Algebra;
import uacalc.util.Project;

import uacalc.ex.VectorListWriteException;
import uacalc.ex.BadVectorListException;

import java.math.*;
import java.text.*;
import java.util.*;
import java.io.*;
import javax.swing.*;

/**
 * This represents a list of vectors.
 * The vectors themselves are represented as int[].
 * Since these cannot be put into HashSets, convert
 * them to IntArray before doing so.
 */

public class VectorList extends ArrayList {

    public VectorList() {
	m_length = 0;
    }

    /**
     * It is the user's responsibility to make sure
     * that Collection c contains int[] objects of
     * equal length, and also to call this.setLength()
     */
    public VectorList(Collection c) {
	super(c);
	m_length = 0;
    }

    public String toString() {
	//Writes the vectors to a String.
	//Is there a convenient way to unify this with Write()?
	//In other words, is there a sprintf in Java?

	StringBuffer sb = new StringBuffer("");
	Iterator it = iterator();
	int[] next;
	int i = 0;
	int j;
	while( it.hasNext() ) {
	    next = (int[])it.next();
	    StringBuffer buf = new StringBuffer("");
	    buf.append(i);
	    i++;
	    StringBuffer spaces = new StringBuffer("");
	    for( j = buf.length(); 
		 j < ewkInputStream.NUM_LEN_OUT_LIMIT; j++ ) {
		spaces.append(" ");
	    }
	    sb.append(spaces);
	    sb.append(buf);
	    sb.append(" : ");
	    sb.append(IntArray.intArrayToString(next));
	    sb.append("\n");
	}
	return sb.toString();
    }

    public void Write(PrintStream fp, int type) 
	throws VectorListWriteException {
	//Writes this VectorList to an already open file.
	//<code>type</code> is used to specify the format.
	//This is only for compatibility with Matt's program
	//(and consequently, uab).
	//<code>type</code>  can be
	//   Project.ConListType
	//     first line: <length of vectors>,any number
	//     subsequent lines: ,a,b,c,... maps in Matt's format
	//   Project.TypeListType
	//     first line:<number of triplets>
	//     second line:empty:
	//     subsequent lines: ,a,b,c means typ(a,b)=c
	// Any other value writes in vectorlist format:
	//     first line:<number of vectors>,<length of vectors>
	//     second line:empty:
	//     subsequent lines: a,b,c,... (no comma at the beginning!)


	if(m_length == 0) {
	    throw new VectorListWriteException(
		"VectorList to write is not initialized.");
	}
	if(type == Project.ConListType ) {
	    fp.println(m_length + "," + 0);//second line not empty.
	} else if( type == Project.TypeListType ) {
	    fp.println(size() + "\n");//Empty second line.
	} else {
	    fp.println(size() + "," + m_length +"\n");//Empty second line.
	}
	if( fp.checkError() ) {
	    throw new VectorListWriteException(
		"Error writing the size of the vectorlist.");
	}

	Iterator it = iterator();
	int[] next;
	int i;
	while( it.hasNext() ) {
	    next = (int[])it.next();
	    for( i=0; i<m_length; i++) {
		if( i == 0 ) {
		    if( type == Project.ConListType || 
			type == Project.TypeListType) {
			fp.print(",");
		    }
		    fp.print(next[i]);
		} else {
		    fp.print("," + next[i]);
		}
	    }
	    fp.println();
	    if( fp.checkError() ) {
		throw new VectorListWriteException(
		    "Error writing the elements of the vectorlist.");
	    }
	}
    }

    public void Read(ewkInputStream fp, int limit, int type)
	throws BadVectorListException {
	//Reads VectorList from an already open file.
	//Checks if entries are < limit.
	//If limit < 0 then no check is made.
	//The meaning of <code>type</code> is the same as in Write().

	int length;
	int size = -1;

	int first = fp.get_integer(",\n\r");
	if( first < 0) {
	    throw new BadVectorListException(
		"Error reading first entry (bad delimiter?).");
	}

	int second = 0;
	if(type != Project.TypeListType ) {
	    second = fp.get_integer("\n\r");
	    if( first < 0) {
		throw new BadVectorListException(
		    "Error reading second entry (bad delimiter?).");
	    }
	}

	if(type == Project.ConListType ) {
	    //size is unknown
	    length = first;
	} else if(type == Project.TypeListType ) {
	    size = first;
	    length = 3;
	    fp.skipTo("\n\r");
	} else {//any other VectorList file
	    size = first;
	    length = second;
	    fp.skipTo("\n\r");
	}

	if( length <=0 || length >= VEC_LEN_LIMIT) {
	    throw new BadVectorListException(
		"Illegal length " + length +
		" of vectors (should be in ["
		+ 1 + "," + (VEC_LEN_LIMIT-1) + "]).");
	}
	m_length = length;

	clear();
	int j, inp;
	int[] next;
	while( true ) {//go through lines
	    if(type == Project.ConListType || 
	       type == Project.TypeListType ) {
		if( fp.skipTo(",") == -1 ) break;
	    }
	    next = new int[m_length];
	    for(j = 0; j<m_length-1; j++) {
                inp = fp.get_integer(",\n\r");
		if( inp == -2 ) break; //needed in case length == 1
                if( inp < 0) {
                    throw new BadVectorListException(
                        "Error reading vector entries (bad delimiter?).");
                }
		if( limit>=0 && inp >=limit) {
		    throw new BadVectorListException(
			"Illegal vector entry " + inp + 
			" (should be in ["
			+ 0 + "," + (limit-1) + "]).");
		}
		next[j]=inp;
	    }
            inp = fp.get_integer("\n\r");
	    if( inp == -2 ) break;
	    if( inp < 0) {
                throw new BadVectorListException(
                    "Error reading last vector entry (bad delimiter?).");
            }
            if( limit>=0 && inp >=limit)
            {
                throw new BadVectorListException(
                    "Illegal vector entry " + inp +
                    " (should be in ["
                    + 0 + "," + (limit-1) + "]).");
            }
            next[m_length-1]=inp;
	    add(next);
	}
	if( type !=  Project.ConListType && size() != size ) {
            throw new BadVectorListException(
                "VectorList file length mismatch.");
	}
    }

    public boolean ReadVectorList(String name, int limit, int type) {
	//See the meaning of limit and type in Read().

        try {
            ewkInputStream fp = new ewkInputStream(
                new FileInputStream(name));
            try {
                Read(fp, limit, type);
            }
            catch(BadVectorListException e) {
                System.out.println(e.toString());
                fp.close();
                return(false);
            }
            fp.close();
        }
        catch(Exception e) {
            System.out.println(e.toString());
            return(false);
        }
        return(true);
    }

    public boolean Input(JFrame parent, int[] limits,
			 String message) {

	if(m_length == 0) return(false);
	VectorReader reader = 
	    new VectorReader(parent, limits, this, message);
	if(  reader.isCancelled()) {
	    //User already confirmed cancel
	    m_length = 0;
	    return(false);
	}
	return(true);
    }

    public int getLength() {
	return m_length;
    }

    public void setLength(int length) {
	m_length = length;
    }

    public static VectorList directProduct(int[] components) 
	throws BadVectorListException {

	int len = components.length;
	BigInteger bigPow = BigInteger.valueOf( (long)1 );
	int i;
	for( i=0; i < len; i++) {
	    bigPow =  bigPow.multiply(
		BigInteger.valueOf( (long)(components[i] )));
	    if( bigPow.compareTo((BigInteger.valueOf(
		(long)VEC_NUM_LIMIT )) ) >= 0 )  {
		throw new BadVectorListException(
		    "Cannot create product universe:\n" +
		    "size should be at most" + (VEC_NUM_LIMIT-1) + ".");
	    }
	}
	int size = bigPow.intValue();
	VectorList prod = new VectorList();
	prod.setLength( len );

	int j, tmp;
	boolean flag;

	int[] p = new int[len];
	for( j=0; j<len; j++) {
	    p[j] = 0;
	}
	prod.add(p);
	int[] q;

	for( i=1; i<size; i++) {
	    p = (int[])prod.get(i-1);
	    q = new int[len];
	    flag = false;
	    for( j=len-1; j >= 0; j--) {
		tmp= p[j];
		if( flag ) {            /* already incremented */
		    q[j] = tmp;
		} else {
		    if( tmp==components[j]-1) {
			q[j] = 0;
		    } else {
			q[j] = tmp+1;
			flag=true;          /* copy the rest */
		    }
		}
	    }
	    prod.add(q);
	}
	return prod;
    }

    protected int m_length;
    public static final int VEC_LEN_LIMIT 
    = Algebra.ALG_SIZE_LIMIT;
    public static final int VEC_NUM_LIMIT
    = Algebra.ALG_SIZE_LIMIT;

}
